/*
 * dsp_manager.h
 *
 *  Created on: 2016/09/20
 *      Author: toa29548
 */

#pragma once

#include <stdint.h>
#include <stddef.h>
#include <dsp_lib/include/dsp/dsp_module.h>
#include <dsp_lib/include/misc/null_check.h>
#include <dsp_lib/include/misc/dsp_const.h>

namespace dsp_lib {

class DspManager {

public:
	// -------------------------------------------------------------
	// 定数
	// -------------------------------------------------------------

	static const uint32_t	DSP_MODULE_ID_NOP				= 0;
	static const uint32_t	DSP_MODULE_ID_THROUGH			= 1;
	static const uint32_t	DSP_MODULE_ID_GAIN				= 2;
	static const uint32_t	DSP_MODULE_ID_MATRIX			= 3;
	static const uint32_t	DSP_MODULE_ID_IIR				= 4;
	static const uint32_t	DSP_MODULE_ID_PEAK				= 5;
	static const uint32_t	DSP_MODULE_ID_RMS				= 6;
	static const uint32_t	DSP_MODULE_ID_COMP				= 7;
	static const uint32_t	DSP_MODULE_ID_GATE				= 8;
	static const uint32_t	DSP_MODULE_ID_DELAY				= 9;
	static const uint32_t	DSP_MODULE_ID_AEC				= 10;
	static const uint32_t	DSP_MODULE_ID_NC				= 11;
	static const uint32_t	DSP_MODULE_ID_MICROPHONEARRAY	= 12;

	// -------------------------------------------------------------
	// データ型
	// -------------------------------------------------------------

	struct NetList {
		uint32_t	type;
		int32_t		input;
		int32_t		output;
		void*		config;
		void*		param;
		void*		status;
		uint32_t	reserved_0;	//!< DSPが入力チャンネル数管理に使用
		uint32_t	reserved_1;	//!< DSPが出力チャンネル数管理に使用
	};

	struct Config {
		NetList*	net_list;			//!< 実行する信号処理のリスト
		uint32_t	net_list_size;		//!< 実行する信号処理の総数
		uint32_t	buffer_size;		//!< 信号バッファの個数
	};

	struct Param {
		uint32_t	ready;			//!< CPUによる初期パラメータ取り込み要求
	};

	struct Status {
		uint32_t	ready_flag;		//!< DSPによる初期パラメータ取り込み完了フラグ
	};

	// -------------------------------------------------------------
	// コンフィギュレーション（実行時に変化しないパラメータ）
	// -------------------------------------------------------------

	/*!
	 * DspManagerコンフィギュレーションの初期化
	 * @note
	 * @param config_ DspManagerのコンフィギュレーション構造体
	 * @param net_list ネットリストへのポインタ
	 * @param net_list_size_ ネットリストのサイズ
	 */
	static bool
	Configure(
		Config*		config_,
		NetList*	net_list_,
		uint32_t	net_list_size_,
		uint32_t	buffer_size_
	)
	{
		if (NullCheck(config_, net_list_) == false) {
			return false;
		}

		config_->net_list_size	= net_list_size_;
		config_->net_list		= net_list_;

		//for (int no = 0; no < config_->net_list_size; no++)
		//{
		//	net_list_[no].type		= DSP_MODULE_ID_NOP;
		//	net_list_[no].input		= NULL;
		//	net_list_[no].output	= NULL;
		//	net_list_[no].config	= NULL;
		//	net_list_[no].param		= NULL;
		//	net_list_[no].status	= NULL;
		//	net_list_[no].reserved_0	= NULL;
		//	net_list_[no].reserved_1	= NULL;
		//}

		if (buffer_size_ <= 0) {
			return false;
		}

		config_->buffer_size = buffer_size_;

		return true;
	}

	/*!
	 * DspManagerコンフィギュレーションの設定
	 * @param config_	DspManagerのコンフィギュレーション構造体
	 * @param no_		信号処理の実行順
	 * @param type_		信号処理の種類
	 * @param input_	入力信号のアドレス
	 * @param output_	出力信号のアドレス
	 * @param param_	Param構造体のアドレス
	 * @param status_	Status構造体のアドレス
	 * @param data_		Data構造体のアドレス
	 * @return
	 */
	static bool
	Setup(
		Config*		dsp_manager_config_,
		uint32_t	no_,
		uint32_t	type_,
		int32_t		input_,
		int32_t		output_,
		void*		config_,
		void*		param_,
		void*		status_
	)
	{
		if (NullCheck(dsp_manager_config_) == false) {
			return false;
		}
		if (no_ >= dsp_manager_config_->net_list_size) {
			return false;
		}
		dsp_manager_config_->net_list[no_].type			= type_;
		dsp_manager_config_->net_list[no_].input		= input_;
		dsp_manager_config_->net_list[no_].output		= output_;
		dsp_manager_config_->net_list[no_].config		= config_;
		dsp_manager_config_->net_list[no_].param		= param_;
		dsp_manager_config_->net_list[no_].status		= status_;
		dsp_manager_config_->net_list[no_].reserved_0	= 0;
		dsp_manager_config_->net_list[no_].reserved_1	= 0;

		return true;
	}

	// -------------------------------------------------------------
	// パラメータ変更
	// -------------------------------------------------------------

	/*!
	 * 初期パラメータ取り込みを依頼する
	 */
	static bool
	Ready(
		Param* param_
	)
	{
		if (NullCheck(param_) == false) {
			return false;
		}
		param_->ready = 1;
		return true;
	}

	// -------------------------------------------------------------
	// ステータス取得
	// -------------------------------------------------------------

	/*!
	 * 初期パラメータ取り込み完了を確認する
	 */
	static bool
	CheckReady(
		Status*	status_
	)
	{
		if (NullCheck(status_) == false) {
			return false;
		}

		if (status_->ready_flag == 1)
		{
			return true;
		} else {
			return false;
		}
	}

public:
	DspManager(void);

	bool Initialize(
			Config*	config_,
			Param*	param_,
			Status*	status_
	);
	bool Execute(
			Param*	param_,
			Status*	status_
	);
	bool Task(
			Param*	param_,
			Status*	status_
	);

	typedef float 		IoBuffer[DSP_BLOCK_SIZE];
	static IoBuffer*	m_io_buffer;				//!< 信号バッファ

protected:
	void PrintNetList(
			uint32_t dsp_no_
	);
//	struct DspBlock {
//		DspModule*	module;
//		float*			input;
//		float*			output;
//	};
	struct IoBufferAddress {
		float*	input;
		float*	output;
	};


	Config*		m_config;
	Param*		m_param;
	Status*		m_status;

	static DspModule**		m_dsp_modules;
	static IoBufferAddress*	m_io_buffer_address;

	static uint32_t		m_io_buffer_size;

	static uint32_t		m_net_list_size;

};

} // namespace dsp_lib
