/*
 * smoothing.h
 *
 *  Created on: 2016/11/28
 *      Author: toa29548
 */

#pragma once

#include <dsp_lib/include/misc/dsp_const.h>
#include <stdint.h>

namespace dsp_lib {

/*!
 * 時間を忘却係数に変換する
 * @param time_				アタックタイム or リリースタイム[sec]
 * @param update_frequency_	データの更新周波数。サンプル更新の場合はサンプリング周波数を, 割込みごとに更新する場合は割込み周波数を指定する
 */
inline float
TimeToForgetFactor(
	const float time_,
	const float	update_frequency_
)
{
	float forget_factor = 1.0f - time_/(time_+(1.0f/(update_frequency_)));	// 増幅時の忘却係数
	return forget_factor;
}

/*!
 * 忘却係数を時間に変換する
 * @param forget_factor_	忘却係数
 * @param update_frequency_	データの更新周波数。サンプル更新の場合はサンプリング周波数を, 割込みごとに更新する場合は割込み周波数を指定する
 */
inline float
ForgetFactorToTime(
	const float forget_factor_,
	const float	update_frequency_
)
{
	float time = (1.0f - forget_factor_) / (update_frequency_ * forget_factor_);
	return time;
}


/*!
 * 時間をサンプル数に変換する
 * @param time_				時間[sec]
 * @param samples_			サンプル[samples]
 */
inline void
TimeToSample(
	const float time_,
	uint32_t*	samples_
)
{
	*samples_ = static_cast<uint32_t>(time_ * DSP_SAMPLING_FREQ);
}


/*!
 * サンプル数を時間に変換する
 * @param samples_			サンプル[samples]
 * @param time_				時間[sec]
 */
inline void
SampleToTime(
	const uint32_t	samples_,
	float*			time_
)
{
	*time_ = samples_ / DSP_SAMPLING_FREQ;
}

/*!
 * 時間をブロック数に変換する
 * @param time_				時間[sec]
 * @param blocks_			ブロック[blocks]
 */
inline void
TimeToBlock(
	const float time_,
	uint32_t*	blocks_
)
{
	*blocks_ = static_cast<uint32_t>(time_ * DSP_BLOCK_FREQ);
}


/*!
 * ブロック数を時間に変換する
 * @param blocks_			ブロック[blocks]
 * @param time_				時間[sec]
 */
inline void
BlockToTime(
	const uint32_t	blocks_,
	float*			time_
)
{
	*time_ = blocks_ / DSP_BLOCK_FREQ;
}

} //namespace dsp_lib
