/*
 * spi_driver.cpp
 */

//********************************************************************************
#include "spi_driver.h"

#if SPI_ENABLE

#include <linux/spi/spidev.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdint.h>
#include <stdio.h>
#include <unistd.h>

//********************************************************************************
//#define DEBUG_SPI_DRIVER

//--------------------------------------------------------------------------------
#define _BYTE1(x) (  x        & 0xFF )
#define _BYTE2(x) ( (x >>  8) & 0xFF )
#define _BYTE3(x) ( (x >> 16) & 0xFF )
#define _BYTE4(x) ( (x >> 24) & 0xFF )

#define BYTE_SWAP_32(x) ((uint32_t)( _BYTE1(x)<<24 | _BYTE2(x)<<16 | _BYTE3(x)<<8 | _BYTE4(x) ))

//********************************************************************************
static const char		SPI_DEVICE0[]			= "/dev/spidev0.0";
static const char		SPI_DEVICE1[]			= "/dev/spidev0.1";
static const uint32_t	SPI_SPEED				= 500000;				// max speed [Hz]
static const uint8_t	SPI_BITS				= 8;					// bits per word
static const uint16_t	SPI_DELAY				= 0;					// [usec]

//--------------------------------------------------------------------------------
static const uint32_t	SPI_NUM_OF_RETRY		= 8*3;
static const uint32_t	SPI_INTERVAL_OF_RETRY	= 1000;					// [usec]

//--------------------------------------------------------------------------------
static const uint32_t	SPI_TRANSFER_LEN		= 6;

static const uint32_t	SPI_CMD_W				= 0x434D4457;
static const uint32_t	SPI_CMD_R				= 0x434D4452;

static const uint32_t	SPI_TX_POS_CMD			= 0;
static const uint32_t	SPI_TX_POS_ADDRESS		= 1;
static const uint32_t	SPI_TX_POS_DATA			= 2;

static const uint32_t	SPI_RX_POS_CMD			= 2;
static const uint32_t	SPI_RX_POS_ADDRESS		= 3;
static const uint32_t	SPI_RX_POS_DATA			= 4;
static const uint32_t	SPI_RX_POS_ACK			= 5;

//********************************************************************************
bool
SpiTransfer(
	const uint32_t*	tx_buf_,
	uint32_t*		rx_buf_,
	const uint32_t	dev_num_
)
{

	//----------------------------------------
	int		fd;
	int		ret;

	//----------------------------------------
	const char*	device_name	= (dev_num_ == 0) ? SPI_DEVICE0 : SPI_DEVICE1;
	uint8_t		mode		= SPI_MODE_0;
	uint8_t		bits		= SPI_BITS;
	uint32_t	speed		= SPI_SPEED;

	fd = open(device_name, O_RDWR);
	if (fd < 0) {
		printf("spi_driver : device open error\n");
		return false;
	}

	ret = ioctl(fd, SPI_IOC_WR_MODE, &mode);
	if (ret == -1) {
		printf("spi_driver : device wr mode error\n");
		close(fd);
		return false;
	}

	ret = ioctl(fd, SPI_IOC_RD_MODE, &mode);
	if (ret == -1) {
		printf("spi_driver : device rd mode error\n");
		close(fd);
		return false;
	}

	ret = ioctl(fd, SPI_IOC_WR_BITS_PER_WORD, &bits);
	if (ret == -1) {
		printf("spi_driver : device wr bits_per_word error\n");
		close(fd);
		return false;
	}

	ret = ioctl(fd, SPI_IOC_RD_BITS_PER_WORD, &bits);
	if (ret == -1) {
		printf("spi_driver : device rd bits_per_word error\n");
		close(fd);
		return false;
	}

	ret = ioctl(fd, SPI_IOC_WR_MAX_SPEED_HZ, &speed);
	if (ret == -1) {
		printf("spi_driver : device wr speed error\n");
		close(fd);
		return false;
	}

	ret = ioctl(fd, SPI_IOC_RD_MAX_SPEED_HZ, &speed);
	if (ret == -1) {
		printf("spi_driver : device rd speed error\n");
		close(fd);
		return false;
	}

	//----------------------------------------
	uint32_t				tx_buf[SPI_TRANSFER_LEN];
	uint32_t				rx_buf[SPI_TRANSFER_LEN];
	struct spi_ioc_transfer	packet;

	for (int i = 0; i < SPI_TRANSFER_LEN; i++) {
		tx_buf[i] = BYTE_SWAP_32(tx_buf_[i]);
	}

	packet.tx_buf			= (unsigned long)tx_buf;
	packet.rx_buf			= (unsigned long)rx_buf;
	packet.len				= sizeof(uint32_t)*SPI_TRANSFER_LEN;
	packet.delay_usecs		= SPI_DELAY;
	packet.speed_hz			= SPI_SPEED;
	packet.bits_per_word	= SPI_BITS;
	packet.cs_change		= 0;
	packet.tx_nbits			= 0;
	packet.rx_nbits			= 0;
	packet.pad				= 0;

	for (uint32_t i=0; i<SPI_NUM_OF_RETRY; i++) {
		ret = ioctl(fd, SPI_IOC_MESSAGE(1), &packet);
		if (ret < 1) {
			printf("spi_driver : can't send spi packet\n");
			close(fd);
			return false;
		}

		for (int i = 0; i < SPI_TRANSFER_LEN; i++) {
			rx_buf_[i] = BYTE_SWAP_32(rx_buf[i]);
		}

		if (
				(rx_buf_[SPI_RX_POS_CMD]		== tx_buf_[SPI_TX_POS_CMD])
			&&	(rx_buf_[SPI_RX_POS_ADDRESS]	== tx_buf_[SPI_TX_POS_ADDRESS])
			&&	(rx_buf_[SPI_RX_POS_ACK]		== rx_buf_[SPI_RX_POS_CMD] + rx_buf_[SPI_RX_POS_ADDRESS] + rx_buf_[SPI_RX_POS_DATA])
		) {
#ifdef DEBUG_SPI_DRIVER
			for (int j=0; j<SPI_TRANSFER_LEN; j++) {
				printf("rx_buf[%d]: 0x%08x\n", j, rx_buf_[j]);
			}
#endif			
			close(fd);
			return true;
		}

		usleep(SPI_INTERVAL_OF_RETRY);
	}

	//----------------------------------------
#ifdef DEBUG_SPI_DRIVER
	for (int j=0; j<SPI_TRANSFER_LEN; j++) {
		printf("rx_buf[%d]: 0x%08x\n", j, rx_buf_[j]);
	}
#endif			
    printf("spi_driver : spi packet ack error\n");
	close(fd);
    return false;
}

//--------------------------------------------------------------------------------
bool
SpiWrite(
	const uint32_t	address_,
	const uint32_t	data_,
	const uint32_t	dev_num_
)
{
	//----------------------------------------
	uint32_t	tx_buf[SPI_TRANSFER_LEN] = {};
	uint32_t	rx_buf[SPI_TRANSFER_LEN] = {};

	tx_buf[SPI_TX_POS_CMD]		= SPI_CMD_W;
	tx_buf[SPI_TX_POS_ADDRESS]	= address_;
	tx_buf[SPI_TX_POS_DATA]		= data_;

	//----------------------------------------
	return SpiTransfer(tx_buf, rx_buf, dev_num_);
}

//--------------------------------------------------------------------------------
bool
SpiRead(
	const uint32_t	address_,
	uint32_t*		data_,
	const uint32_t	dev_num_
)
{
	//----------------------------------------
	uint32_t	tx_buf[SPI_TRANSFER_LEN] = {};
	uint32_t	rx_buf[SPI_TRANSFER_LEN] = {};

	tx_buf[SPI_TX_POS_CMD]		= SPI_CMD_R;
	tx_buf[SPI_TX_POS_ADDRESS]	= address_;

	//----------------------------------------
	bool ret = SpiTransfer(tx_buf, rx_buf, dev_num_);

	if (ret == true) {
		*data_ = rx_buf[SPI_RX_POS_DATA];
	}

	return ret;
}
#else
bool
SpiWrite(
	const uint32_t	address_,
	const uint32_t	data_,
	const uint32_t	dev_num_
){
	return true;
};

bool
SpiRead(
	const uint32_t	address_,
	uint32_t*		data_,
	const uint32_t	dev_num_
){
	return true;
}
#endif