/*
 *	dsp_comp.h
 *
 */
#pragma once

#include <stdint.h>
#include <dsp_lib/include/dsp/dsp_module.h>
#include <dsp_lib/include/misc/dsp_const.h>
#include <dsp_lib/include/dsp/peak.h>
#include <dsp_lib/include/dsp/rms.h>

namespace dsp_lib {

class Comp : public DspModule {
public:
	// -------------------------------------------------------------
	// 定数
	// -------------------------------------------------------------
	enum COMP_LEVEL_TYPE {
		COMP_LEVEL_TYPE_PEAK	= 0,
		COMP_LEVEL_TYPE_RMS		= 1
	};

	enum COMP_RATIO {
		COMP_RATIO_1	= 0,
		COMP_RATIO_1_1	= 1,
		COMP_RATIO_1_2  = 2,
		COMP_RATIO_1_3  = 3,
		COMP_RATIO_1_5  = 4,
		COMP_RATIO_1_7  = 5,
		COMP_RATIO_2    = 6,
		COMP_RATIO_2_3  = 7,
		COMP_RATIO_2_6  = 8,
		COMP_RATIO_3    = 9,
		COMP_RATIO_4    = 10,
		COMP_RATIO_5    = 11,
		COMP_RATIO_7    = 12,
		COMP_RATIO_8    = 13,
		COMP_RATIO_10   = 14,
		COMP_RATIO_12   = 15,
		COMP_RATIO_20   = 16,
		COMP_RATIO_INF  = 17
	};

	enum COMP_KNEE {
		COMP_KNEE_HARD 	= 0,
		COMP_KNEE_SOFT1	= 1,
		COMP_KNEE_SOFT2	= 2
	};

	// -------------------------------------------------------------
	// データ型
	// -------------------------------------------------------------
	struct Config{};

	struct Param{
		float		attack;				//!< リダクションをかける際の時定数
		float		release;			//!< リダクションを解除する際の時定数
		float		threshold;			//!< 単位:真数
		float		makeup_gain;		//!< 単位:真数
		uint32_t	ratio;				//!< インデックス指定
		uint32_t	knee;				//!< インデックス指定
		uint32_t	level_type;			//!< Peak or RMS
	};

	struct Status{
		float		gain_target;		//!< ゲインの目標値
		float		gain_current;		//!< ゲインの現在値
		float		peak;				//!< 直近5msecのpeak, 単位:振幅の2乗
		float		peak_release;		//!< peakと同様に上昇するが、ゆっくりと下降する, 単位:振幅の2乗
		float		rms;				//!< サンプルの2乗平均, 平方根は取っていない
	};

	// -------------------------------------------------------------
	// コンフィギュレーション（実行時に変化しないパラメータ設定）
	// -------------------------------------------------------------
	static bool Configure(void) {
		return true;
	};
	static bool Attach(void) {
		return true;
	}

	Comp(
			Config*	/*config_*/,
			Param*	param_,
			Status*	status_
	);

	bool Ready(void);

	bool Execute(
			const float*	input_,
				  float*	output_
	);
	bool Task(void);

protected:
	/*!
	    @brief  コンプレッサーのkneeに関するパラメータ構造体

	    - ゲインダイアグラムで、入力信号がthresholdを上回る部分は3種類の近似式によってゲインを決定する。
	*/
	struct CompresserCoeffData {
		float	ratio;		//!< ratio
		float	coeff_th;	//!< soft kneeの場合にthreshold手前でリダクションをかけ始めるための係数
		float	a0[4];		//!< 入力レベルがthresholdを0~20dB超えた場合に使用する近似式の係数セット
		float	a20[4];		//!< 入力レベルがthresholdを20~40超えた場合に使用する近似式の係数セット
		float	a40[4];		//!< 入力レベルがthresholdを40dB以上超えた場合に使用する近似式の係数セット
	};

	static const CompresserCoeffData*	s_coeff_compresser_table[];
	static const CompresserCoeffData	s_coeff_compresser_hard_knee_table[];
	static const CompresserCoeffData	s_coeff_compresser_soft_knee10_table[];
	static const CompresserCoeffData	s_coeff_compresser_soft_knee20_table[];

	static const int					s_coeff_compresser_size_table[];
	static const int					s_coeff_compresser_size_table_size;

	void execute_comp_calc_level(
			const float*	input_
	);
	void execute_comp_calc_target_gain(void);
	void execute_comp_calc_current_gain(void);
	void execute_comp_gain(
			const float*	input_,
				  float*	output_
	);


//	Config*			m_config;
	Param*			m_param;
	Status*			m_status;

	Peak*			m_peak;
	Peak::Status	m_peak_status;

	Rms*			m_rms;
	Rms::Status		m_rms_status;

	float						m_gain[DSP_BLOCK_SIZE];	//!< TODO:TIは8byte境界に置くこと, 単位:真数
	float						m_gain_current;			//!< 単位:真数
	const CompresserCoeffData*	m_coeff;				//!< ratio, kneeに基づく係数
	float						m_level;				//!< 単位:振幅の2乗
};

} //namespace dsp_lib
