/*************************************
 *  BM64Control.c
 *
 *  History
 *  2017/09/28  1st release
 *************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <strings.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <pthread.h>
#include <errno.h>
#include <time.h>
#include "BM64Control.h"

unsigned int DbgFlag = 0;

const char DumpData1[] = "0x%02x\n";
const char DumpData2[] = "0x%02x 0x%02x\n";
const char DumpData3[] = "0x%02x 0x%02x 0x%02x\n";
const char DumpData4[] = "0x%02x 0x%02x 0x%02x 0x%02x\n";
const char DumpData5[] = "0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n";
const char DumpData6[] = "0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n";
const char DumpData7[] = "0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n";
const char DumpData8[] = "0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x\n";

#if defined(DBG_FLG)
#define DPRINTF(a)  Bm64DPrintf a
#define DDUMP(a)    Bm64DumpData a
#else
#define DPRINTF(a)
#define DDUMP(a)
#endif

int LibInit;
int ComFd;
int ComOpen;
int TWorkerLoop;
int TWorkerOpen;
int SendRequest;
int Sending;
pthread_t TWorker;
unsigned char *pRecvBuf;
unsigned char *pSendBuf;
struct termios OrgTio;
pthread_mutex_t SendMtx = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t SendSyncMtx = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t SendSyncCnd = PTHREAD_COND_INITIALIZER;
BM64_SEND_DATA_INFO SendDataInfo;
LibBM64_Callback pEventCb;

void Bm64DPrintf( unsigned int Flags, const char *Fmt, ... )
{
    char str[BM64_DEBUG_MSG_SIZE];
    va_list ap;

    if ((Flags & DbgFlag) == 0)
    {
        return;
    }

    va_start(ap, Fmt);
    vsprintf(str, Fmt, ap);
    va_end(ap);
    printf(str);
}

void Bm64DumpData( unsigned char *Buffer, unsigned int Size )
{
    unsigned int i;
    unsigned int j;
    unsigned int k;
    unsigned int l;

    if ((DbgFlag & BM64_DBG_DUMP) == 0)
    {
        return;
    }

    if (Size == 0)
    {
        /* サイズ0はダンプの必要が無い */
        return;
    }

    i = Size / 8;
    j = Size % 8;

    printf("----dump size %d----\n", Size);

    l = 0;
    for (k = 0; k <= i; k++)
    {
        if (k == i)
        {
            /* ダンプの最後、端数表示 */
            switch (j)
            {
            case 1:
                printf(DumpData1, Buffer[l]);
                break;
            case 2:
                printf(DumpData2, Buffer[l], Buffer[l + 1]);
                break;
            case 3:
                printf(DumpData3, Buffer[l], Buffer[l + 1], Buffer[l + 2]);
                break;
            case 4:
                printf(DumpData4, Buffer[l], Buffer[l + 1], Buffer[l + 2], Buffer[l + 3]);
                break;
            case 5:
                printf(DumpData5, Buffer[l], Buffer[l + 1], Buffer[l + 2], Buffer[l + 3],
                       Buffer[l + 4]);
                break;
            case 6:
                printf(DumpData6, Buffer[l], Buffer[l + 1], Buffer[l + 2], Buffer[l + 3],
                       Buffer[l + 4], Buffer[l + 5]);
                break;
            case 7:
                printf(DumpData7, Buffer[l], Buffer[l + 1], Buffer[l + 2], Buffer[l + 3],
                       Buffer[l + 4], Buffer[l + 5], Buffer[l + 6]);
                break;
            case 8:
                printf(DumpData8, Buffer[l], Buffer[l + 1], Buffer[l + 2], Buffer[l + 3],
                       Buffer[l + 4], Buffer[l + 5], Buffer[l + 6], Buffer[l + 7]);
                break;
            default:
                break;
            }
        }
        else
        {
            printf(DumpData8, Buffer[l], Buffer[l + 1], Buffer[l + 2], Buffer[l + 3],
                   Buffer[l + 4], Buffer[l + 5], Buffer[l + 6], Buffer[l + 7]);
            l += 8;
        }
    }

    printf("----dump end----\n");
}

unsigned char Bm64SumCalc( unsigned char *pBuffer, unsigned int PacketLength )
{
    unsigned int i;
    unsigned char crc;

    //DPRINTF((BM64_DBG_FUNC_ENTER, "Bm64SumCalc enter.\n"));

    crc = 0;
    for (i = 1; i < (PacketLength - 1); i++)
    {
        crc += pBuffer[i];
    }
    crc = (~crc) + 1;

    //DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64SumCalc exit 0x%02x.\n", crc));
    return(crc);
}

int Bm64SumCheck( unsigned char *pBuffer, unsigned int PacketLength )
{
    unsigned char crc;
    unsigned int i;

    //DPRINTF((BM64_DBG_FUNC_ENTER, "Bm64SumCheck enter.\n"));

    i = PacketLength - 1;
    crc = Bm64SumCalc(pBuffer, PacketLength);
    if (crc == pBuffer[i])
    {
        i = BM64_STATUS_SUCCESS;
    }
    else
    {
        DPRINTF((BM64_DBG_ERROR, "invalid check sum 0x%02x != 0x%02x.\n", crc, pBuffer[i]));
        i = BM64_STATUS_INVALID_CSUM;
    }

    //DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64SumCheck exit %d.\n", i));
    return(i);
}

unsigned short Bm64PayloadLengthGet( unsigned char *pBuffer )
{
    unsigned short tmp;

    //DPRINTF((BM64_DBG_FUNC_ENTER, "Bm64PayloadLengthGet enter.\n"));

    tmp = (unsigned short)pBuffer[1];
    tmp <<= 8;
    tmp = (unsigned short)pBuffer[2];

    //DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64PayloadLengthGet exit %d.\n", tmp));
    return(tmp);
}

void Bm64PayloadLengthSet( unsigned char *pBuffer, unsigned short PayLoadLength )
{
    //DPRINTF((BM64_DBG_FUNC_ENTER, "Bm64PayloadLengthSet enter.\n"));

    pBuffer[1] = (unsigned char)(PayLoadLength >> 8);
    pBuffer[2] = (unsigned char)(PayLoadLength & 0xFF);

    //DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64PayloadLengthSet exit.\n"));
}

void Bm64EventAckBuild( unsigned char *pBuffer, unsigned char OpCode )
{
    DPRINTF((BM64_DBG_FUNC_ENTER, "Bm64EventAckBuild enter.\n"));

    pBuffer[0] = 0xAA;
    pBuffer[1] = 0x00;
    pBuffer[2] = 0x02;
    pBuffer[3] = 0x14;
    pBuffer[4] = OpCode;
    pBuffer[5] = Bm64SumCalc(pBuffer, BM64_EVENT_ACK_SIZE);

    DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64EventAckBuild exit.\n"));
}

int Bm64SendProcess( unsigned char *pBuffer, unsigned int Length )
{
    int ret;
    struct timespec timeout;

    DPRINTF((BM64_DBG_FUNC_ENTER, "Bm64SendProcess enter.\n"));

    if (LibInit == 0)
    {
        DPRINTF((BM64_DBG_ERROR, "Library is not initialized.\n"));
        DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64SendProcess exit %d.\n", BM64_STATUS_NOT_INITIALIZE));
        return(BM64_STATUS_NOT_INITIALIZE);
    }

    DPRINTF((BM64_DBG_LOCK, "SendMtx on LINE:%d.\n", __LINE__));
    pthread_mutex_lock(&SendMtx);

    if (Sending)
    {
        DPRINTF((BM64_DBG_LOCK, "SendMtx off L:%d.\n", __LINE__));
        pthread_mutex_unlock(&SendMtx);

        DPRINTF((BM64_DBG_ERROR, "send processing is working in other thread.\n"));

        DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64SendProcess exit %d.\n", BM64_STATUS_SEND_BUSY));
        return(BM64_STATUS_SEND_BUSY);
    }
    Sending = 1;

    SendDataInfo.Request = 1;
    SendDataInfo.Complete = 0;
    SendDataInfo.Length = Length;
    SendDataInfo.OpCode = pBuffer[BM64_INDEX_OPCODE];
    SendDataInfo.Status = BM64_STATUS_SUCCESS;
    memcpy(pSendBuf, pBuffer, Length);

    DPRINTF((BM64_DBG_LOCK, "SendMtx off L:%d.\n", __LINE__));
    pthread_mutex_unlock(&SendMtx);

    if (SendDataInfo.Complete == 0)
    {
        timeout.tv_sec = time(NULL) + BM64_SEND_COMMAND_TIMEOUT;
        timeout.tv_nsec = 0;

        DPRINTF((BM64_DBG_LOCK, "SendSyncCnd on L:%d.\n", __LINE__));
        ret = pthread_cond_timedwait(&SendSyncCnd, &SendSyncMtx, &timeout);
        DPRINTF((BM64_DBG_LOCK, "SendSyncCnd off L:%d.\n", __LINE__));

        if (ret == 0)
        {
            DPRINTF((BM64_DBG_COMMENT, "SendSyncCnd signal detect.\n"));
        }
        else
        {
            if (ret == ETIMEDOUT)
            {
                DPRINTF((BM64_DBG_ERROR, "SendSyncCnd timeout.\n"));

                DPRINTF((BM64_DBG_LOCK, "SendMtx on L:%d.\n", __LINE__));
                pthread_mutex_lock(&SendMtx);

                SendDataInfo.Request = 0;
                SendDataInfo.Complete = 1;
                SendDataInfo.Status = BM64_ACK_NOT_RESPONSE;

                DPRINTF((BM64_DBG_LOCK, "SendMtx off L:%d.\n", __LINE__));
                pthread_mutex_unlock(&SendMtx);
            }
            else
            {
                DPRINTF((BM64_DBG_ERROR, "SendSyncCnd error:%d.\n", ret));

                DPRINTF((BM64_DBG_LOCK, "SendMtx on L:%d.\n", __LINE__));
                pthread_mutex_lock(&SendMtx);

                SendDataInfo.Request = 0;
                SendDataInfo.Complete = 0;

                DPRINTF((BM64_DBG_LOCK, "SendMtx off L:%d.\n", __LINE__));
                pthread_mutex_unlock(&SendMtx);
            }
        }
    }
    else
    {
        DPRINTF((BM64_DBG_COMMENT, "Command has already completed.\n"));
    }

    if (SendDataInfo.Complete)
    {
        switch (SendDataInfo.Status)
        {
        case BM64_ACK_COMPLETE:
            ret = BM64_STATUS_SUCCESS;
            break;
        case BM64_ACK_DISALLOW:
            ret = BM64_STATUS_DISALLOW;
            break;
        case BM64_ACK_UNKNOWN_COMMAND:
            ret = BM64_STATUS_UNKNOWN;
            break;
        case BM64_ACK_PARAM_ERROR:
            ret = BM64_STATUS_PARAM_ERROR;
            break;
        case BM64_ACK_BUSY:
            ret = BM64_STATUS_BTM_BUSY;
            break;
        case BM64_ACK_MEMORY_FULL:
            ret = BM64_STATUS_BTM_FULL;
            break;
        case BM64_ACK_WRITE_ERROR:
            ret = BM64_STATUS_WRITE_ERROR;
            break;
        case BM64_ACK_NOT_RESPONSE:
            ret = BM64_STATUS_NOT_RESPONSE;
            break;
        default:
            ret = BM64_STATUS_UNSUCCESSFUL;
            break;
        }
    }
    else
    {
        DPRINTF((BM64_DBG_ERROR, "Send Command Failed.\n"));
        ret = BM64_STATUS_UNSUCCESSFUL;
    }

    DPRINTF((BM64_DBG_LOCK, "SendMtx on LINE:%d.\n", __LINE__));
    pthread_mutex_lock(&SendMtx);

    Sending = 0;

    DPRINTF((BM64_DBG_LOCK, "SendMtx off L:%d.\n", __LINE__));
    pthread_mutex_unlock(&SendMtx);

    DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64SendProcess exit %d.\n", ret));
    return(ret);
}

int Bm64LibDeinit( void )
{
    void *tRet;

    DPRINTF((BM64_DBG_FUNC_ENTER, "Bm64LibDeinit enter.\n"));

    pEventCb = NULL;

    if (TWorkerOpen)
    {
        TWorkerLoop = 0;

        DPRINTF((BM64_DBG_ALLOC, "TWorker join.\n"));
        pthread_join(TWorker, &tRet);
        DPRINTF((BM64_DBG_ALLOC, "TWorker join returned 0x%x.\n", tRet));

        TWorker = 0;
        TWorkerOpen = 0;
    }

    if (ComOpen)
    {
        DPRINTF((BM64_DBG_COMMENT, "restore com settings.\n"));
        tcsetattr(ComFd, TCSANOW, &OrgTio);

        DPRINTF((BM64_DBG_ALLOC, "com(fd:0x%x) close.\n", ComFd));
        close(ComFd);
        ComFd = -1;
        ComOpen = 0;
    }

    if (pRecvBuf)
    {
        DPRINTF((BM64_DBG_ALLOC, "free pRecvBuf:0x%x.\n", pRecvBuf));
        free(pRecvBuf);
        pRecvBuf = NULL;
    }

    if (pSendBuf)
    {
        DPRINTF((BM64_DBG_ALLOC, "free pSendBuf:0x%x.\n", pSendBuf));
        free(pSendBuf);
        pSendBuf = NULL;
    }

    DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64LibDeinit exit %d.\n", BM64_STATUS_SUCCESS));
    return(BM64_STATUS_SUCCESS);
}

int Bm64RecvProcess( unsigned char *pBuffer, unsigned int Length )
{
    int ret;
    int sig;
    unsigned char opCode;
    unsigned char eventAck[BM64_EVENT_ACK_SIZE];

    DPRINTF((BM64_DBG_FUNC_ENTER, "Bm64RecvProcess enter.\n"));

    DDUMP((pBuffer, Length));

    if (Bm64SumCheck(pBuffer, Length) < 0)
    {
        DPRINTF((BM64_DBG_ERROR, "Recv:Crc Error.\n"));
        DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64RecvProcess exit %d.\n", BM64_STATUS_INVALID_CSUM));
        return(BM64_STATUS_INVALID_CSUM);
    }

    ret = BM64_STATUS_SUCCESS;
    sig = 0;

    opCode = pBuffer[BM64_INDEX_OPCODE];
    DPRINTF((BM64_DBG_EVENT, "Recv:OpCode 0x%02x.\n", opCode));

    if (opCode == BM64_OPCODE_EV_COMMAND_ACK)
    {
        DPRINTF((BM64_DBG_ACK,
                 "Recv:Command ACK:CmdId:0x%02x STATUS:0x%02x.\n",
                 pBuffer[BM64_INDEX_COMMAND_OPCODE], pBuffer[BM64_INDEX_STATUS]));

        DPRINTF((BM64_DBG_LOCK, "SendMtx on L:%d.\n", __LINE__));
        pthread_mutex_lock(&SendMtx);

        if ((SendDataInfo.Request) && (SendDataInfo.Length == 0))
        {
            if (SendDataInfo.OpCode == pBuffer[BM64_INDEX_COMMAND_OPCODE])
            {
                SendDataInfo.Status = pBuffer[BM64_INDEX_STATUS];
                SendDataInfo.Request = 0;
                SendDataInfo.Complete = 1;
                sig = 1;
            }
        }

        DPRINTF((BM64_DBG_LOCK, "SendMtx off L:%d.\n", __LINE__));
        pthread_mutex_unlock(&SendMtx);

        if (sig)
        {
            DPRINTF((BM64_DBG_LOCK, "SendSyncCnd signal.\n"));
            ret = pthread_cond_signal(&SendSyncCnd);
            if (ret != 0)
            {
                DPRINTF((BM64_DBG_ERROR, "SendSyncCnd signal error %d.\n", ret));
            }
            else
            {
                DPRINTF((BM64_DBG_LOCK, "SendSyncCnd signal complete.\n"));
            }
        }
    }
    else
    {
        if (pEventCb)
        {
            DPRINTF((BM64_DBG_CALLBACK, "Event:0x%x len:%d.\n",
                     pBuffer[BM64_INDEX_OPCODE], (int)(Length - BM64_PACKET_APPENDIX)));
            pEventCb(&pBuffer[BM64_INDEX_OPCODE], (int)(Length - BM64_PACKET_APPENDIX));
        }
        else
        {
            DPRINTF((BM64_DBG_CAUTION, "Recv:Not support event:CmdId:0x%02x.\n", opCode));
            //DDUMP((pBuffer, Length));
        }

        Bm64EventAckBuild(eventAck, opCode);
        //DDUMP((eventAck, BM64_EVENT_ACK_SIZE));

        DPRINTF((BM64_DBG_ACK, "Send:Event ACK:CmdId:0x%02x.\n",opCode));
        ret = write(ComFd, eventAck, BM64_EVENT_ACK_SIZE);
        if (ret < 0)
        {
            DPRINTF((BM64_DBG_COMMENT, "Send:Event Ack error %d.\n", errno));
            ret = BM64_STATUS_WRITE_ERROR;
        }
        else
        {
            if (ret == BM64_EVENT_ACK_SIZE)
            {
                DPRINTF((BM64_DBG_COMMENT, "Send:Event ACK complete.\n"));
                ret = BM64_STATUS_SUCCESS;
            }
            else
            {
                DPRINTF((BM64_DBG_ERROR, "Send:Event ACK length error %d != %d.\n"
                         , BM64_EVENT_ACK_SIZE, ret));
                ret = BM64_STATUS_WRITE_ERROR;
            }

        }
    }

    DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64RecvProcess exit %d.\n", ret));
    return(ret);
}

int Bm64DataProcess( unsigned char *pBuffer, unsigned int Length )
{
    int ret;
    unsigned int uret;
    int sig;
    unsigned int i;
    unsigned short packetLen;
    unsigned short payLen;
    unsigned char *ptr;

    //DPRINTF((BM64_DBG_FUNC_ENTER, "Bm64DataProcess enter.\n"));

    ptr = pBuffer;
    //DDUMP((ptr, Length));

    i = 0;
    while (i < Length)
    {
        if (*ptr == BM64_SYNC_WORD)
        {
            payLen = Bm64PayloadLengthGet(ptr);
            packetLen = payLen + BM64_PACKET_APPENDIX;

            DPRINTF((BM64_DBG_COMMENT, "Recv:Packet length:%d.\n", packetLen));
            DPRINTF((BM64_DBG_COMMENT, "Recv:PayLoad length:%d.\n", payLen));

            Bm64RecvProcess(ptr, packetLen);

            ptr += packetLen;
            i += packetLen;
        }
        else
        {
            ptr++;
            i++;
        }
    }

    if (Length == 0)
    {
        //data send
        sig = 0;

        DPRINTF((BM64_DBG_LOCK, "SendMtx on L:%d.\n", __LINE__));
        pthread_mutex_lock(&SendMtx);

        if (SendDataInfo.Request)
        {
            if (SendDataInfo.Length)
            {
                DPRINTF((BM64_DBG_SEND, "Send.\n"));
                DDUMP((pSendBuf, SendDataInfo.Length));

                uret = write(ComFd, pSendBuf, SendDataInfo.Length);
                if (uret == SendDataInfo.Length)
                {
                    DPRINTF((BM64_DBG_SEND, "Send complete.\n"));
                    SendDataInfo.Length = 0;
                }
                else
                {
                    DPRINTF((BM64_DBG_ERROR, "Send error. req:%d ret:%d errno:%d.\n",
                             SendDataInfo.Length, ret, errno));
                    SendDataInfo.Length = 0;
                    SendDataInfo.Status = BM64_ACK_WRITE_ERROR;
                    SendDataInfo.Request = 0;
                    SendDataInfo.Complete = 1;
                    sig = 1;
                }
            }
        }

        DPRINTF((BM64_DBG_LOCK, "SendMtx off L:%d.\n", __LINE__));
        pthread_mutex_unlock(&SendMtx);

        if (sig)
        {
            DPRINTF((BM64_DBG_LOCK, "SendSyncCnd signal.\n"));
            ret = pthread_cond_signal(&SendSyncCnd);
            if (ret != 0)
            {
                DPRINTF((BM64_DBG_ERROR, "SendSyncCnd signal error %d.\n", errno));
            }
            else
            {
                DPRINTF((BM64_DBG_LOCK, "SendSyncCnd signal complete.\n"));
            }
        }
    }

    //DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64DataProcess exit %d.\n", BM64_STATUS_SUCCESS));
    return(BM64_STATUS_SUCCESS);
}

#define LOOP_MIN_TIME 100 * 1000 * 1000 //nsec

static void* Bm64WorkerThread( void *Arg )
{
    ssize_t ret;
    unsigned int len;
    unsigned char *ptr;
    unsigned long diff_time;
    struct timespec startTime, endTime, sleepTime;

    DPRINTF((BM64_DBG_FUNC_ENTER, "Bm64WorkerThread enter.\n"));

    len = 0;
    ptr = pRecvBuf;

    startTime.tv_sec = 0;
    startTime.tv_nsec = 0;
    sleepTime.tv_sec = 0;
    sleepTime.tv_nsec = 0;

    while (TWorkerLoop)
    {
        DPRINTF((BM64_DBG_RECV, "Recv.\n"));

        clock_gettime(CLOCK_REALTIME, &startTime);
        //DPRINTF((BM64_DBG_FUNC_ENTER, "startTime %d.\n",startTime.tv_nsec));

        ret = read(ComFd, ptr + len, BM64_RECV_BUFF_SIZE - len);
        if (ret < 0)
        {
            DPRINTF((BM64_DBG_ERROR, "Recv error %d.\n", errno));
        }
        else
        {
            DPRINTF((BM64_DBG_RECV, "Recv complete %d bytes.\n", ret));

            if (ret == 0)
            {
                //Just the end of data.
                Bm64DataProcess(pRecvBuf, len);

                len = 0;
                ptr = pRecvBuf;

                clock_gettime(CLOCK_REALTIME, &endTime);
                //DPRINTF((BM64_DBG_FUNC_ENTER, "endTime %d.\n",endTime.tv_nsec));

                if (endTime.tv_nsec > startTime.tv_nsec) {
                    diff_time = endTime.tv_nsec - startTime.tv_nsec;
                    //DPRINTF((BM64_DBG_FUNC_ENTER, "diff_time %d.\n",diff_time));

                    if(LOOP_MIN_TIME > diff_time){
                        sleepTime.tv_sec = 0;
                        sleepTime.tv_nsec = LOOP_MIN_TIME - diff_time;
                        //DPRINTF((BM64_DBG_FUNC_ENTER, "sleepTime.tv_nsec %d.\n",sleepTime.tv_nsec));
                        nanosleep(&sleepTime, NULL);
                    }
                }
            }
            else
            {
                //Next data may arrive.
                len += (unsigned int)ret;

                if (len >= BM64_RECV_BUFF_SIZE)
                {
                    DPRINTF((BM64_DBG_ERROR, "Recv buffer full.\n"));
                }
            }
        }
    }

    DPRINTF((BM64_DBG_FUNC_EXIT, "Bm64WorkerThread exit %d.\n", BM64_STATUS_SUCCESS));
    return((void *)BM64_STATUS_SUCCESS);
}

int LibBM64_Init( void )
{
    int ret;
    struct termios tio;

    printf("LibBM64 build:%s Time:%s\n", __DATE__, __TIME__);

#if defined(DBG_FLG)
    DbgFlag = DBG_FLG;
#endif

    DPRINTF((BM64_DBG_FUNC_ENTER, "LibBM64_Init enter.\n"));

    if (LibInit)
    {
        DPRINTF((BM64_DBG_ERROR, "LibBM64_Init have already called from App.\n"));
        DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_Init exit %d.\n", BM64_STATUS_UNSUCCESSFUL));
        return(BM64_STATUS_UNSUCCESSFUL);
    }

    pEventCb = NULL;

    pRecvBuf = malloc(BM64_RECV_BUFF_SIZE);
    if (pRecvBuf == NULL)
    {
        DPRINTF((BM64_DBG_ERROR, "pRecvBuf allocate error.\n"));
        Bm64LibDeinit();
        DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_Init exit %d.\n", BM64_STATUS_UNSUCCESSFUL));
        return(BM64_STATUS_UNSUCCESSFUL);
    }
    DPRINTF((BM64_DBG_ALLOC, "alloc pRecvBuf:0x%x.\n", pRecvBuf));

    pSendBuf = malloc(BM64_SEND_BUFF_SIZE);
    if (pSendBuf == NULL)
    {
        DPRINTF((BM64_DBG_ERROR, "pSendBuf allocate error.\n"));
        Bm64LibDeinit();
        DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_Init exit %d.\n", BM64_STATUS_UNSUCCESSFUL));
        return(BM64_STATUS_UNSUCCESSFUL);
    }
    DPRINTF((BM64_DBG_ALLOC, "alloc pSendBuf:0x%x.\n", pSendBuf));

    ComFd = open(BM64_SERIAL_PORT, O_RDWR | O_NOCTTY);
    if (ComFd < 0)
    {
        DPRINTF((BM64_DBG_ERROR, "com open failed 0x%x.\n", ComFd));
        Bm64LibDeinit();
        DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_Init exit %d.\n", BM64_STATUS_DEVICE_ERROR));
        return(BM64_STATUS_DEVICE_ERROR);
    }
    ComOpen = 1;
    DPRINTF((BM64_DBG_ALLOC, "com(fd:0x%x) open.\n", ComFd));

    DPRINTF((BM64_DBG_COMMENT, "backup com settings.\n"));
    tcgetattr(ComFd, &OrgTio);

    bzero(&tio, sizeof(tio));
    tio.c_iflag = BM64_SERIAL_IN_FLAGS;
    tio.c_oflag = BM64_SERIAL_OUT_FLAGS;
    tio.c_cflag = BM64_SERIAL_CTRL_FLAGS;
    tio.c_lflag = BM64_SERIAL_LOCAL_FLAGS;

    tio.c_cc[VTIME] = BM64_SERIAL_TIMEOUT;

    DPRINTF((BM64_DBG_COMMENT, "flash com data.\n"));
    tcflush(ComFd, TCIFLUSH);

    DPRINTF((BM64_DBG_COMMENT, "update com settings.\n"));
    tcsetattr(ComFd, TCSANOW, &tio);

    TWorkerLoop = 1;
    ret = pthread_create(&TWorker, NULL, Bm64WorkerThread, NULL);
    if (ret != 0)
    {
        DPRINTF((BM64_DBG_ERROR, "Bm64WorkerThread create error 0x%x.\n\n", ret));
        Bm64LibDeinit();
        DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_Init exit %d.\n", BM64_STATUS_UNSUCCESSFUL));
        return(BM64_STATUS_UNSUCCESSFUL);
    }
    TWorkerOpen = 1;
    DPRINTF((BM64_DBG_ALLOC, "TWorker:0x%x open.\n", TWorker));

    LibInit = 1;
    DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_Init exit %d.\n", BM64_STATUS_SUCCESS));
    return(BM64_STATUS_SUCCESS);
}

int LibBM64_Deinit( void )
{
    DPRINTF((BM64_DBG_FUNC_ENTER, "LibBM64_Deinit enter.\n"));

    if (LibInit == 0)
    {
        DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_Deinit exit %d.\n", BM64_STATUS_NOT_INITIALIZE));
        return(BM64_STATUS_NOT_INITIALIZE);
    }

    Bm64LibDeinit();

    LibInit = 0;
    DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_Deinit exit %d.\n", BM64_STATUS_SUCCESS));
    return(BM64_STATUS_SUCCESS);
}

int LibBM64_CallbackSet( LibBM64_Callback pCallback )
{
    DPRINTF((BM64_DBG_FUNC_ENTER, "LibBM64_CallbackSet enter.\n"));

    if (LibInit == 0)
    {
        DPRINTF((BM64_DBG_ERROR, "Library is not initialized.\n"));
        DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_CallbackSet exit %d.\n", BM64_STATUS_NOT_INITIALIZE));
        return(BM64_STATUS_NOT_INITIALIZE);
    }

    pEventCb = pCallback;

    DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_CallbackSet exit %d.\n", BM64_STATUS_SUCCESS));
    return(BM64_STATUS_SUCCESS);
}

int LibBM64_EnterPairingMode( void )
{
    ssize_t ret;
    unsigned char buf[BM64_WORK_BUFF_SIZE];

    DPRINTF((BM64_DBG_FUNC_ENTER, "LibBM64_EnterPairingMode enter.\n"));

    //MMI_Action:enter pairing mode (from power off state)
    buf[0] = 0xAA;
    buf[1] = 0x00;
    buf[2] = 0x03;
    buf[3] = 0x02;
    buf[4] = 0x00;
    buf[5] = 0x5D;
    buf[6] = 0x01;
    buf[7] = Bm64SumCalc(buf, 8);

    ret = Bm64SendProcess(buf, 8);
    if (ret < 0)
    {
        DPRINTF((BM64_DBG_COMMENT, "Bm64SendProcess !ERROR! returned FUNC:LibBM64_EnterPairingMode %d.\n", ret));
    }

    DPRINTF((BM64_DBG_FUNC_EXIT, "EnterPairingMode exit %d.\n", ret));
    return(ret);
}

int LibBM64_ExitPairingMode( void )
{
    ssize_t ret;
    unsigned char buf[BM64_WORK_BUFF_SIZE];

    DPRINTF((BM64_DBG_FUNC_ENTER, "ExitPairingMode enter.\n"));

    //MMI_Action:Exit pairing mode
    buf[0] = 0xAA;
    buf[1] = 0x00;
    buf[2] = 0x03;
    buf[3] = 0x02;
    buf[4] = 0x00;
    buf[5] = 0x6B;
    buf[6] = Bm64SumCalc(buf, BM64_MMI_ACTION_SIZE);

    ret = Bm64SendProcess(buf, BM64_MMI_ACTION_SIZE);
    if (ret < 0)
    {
        DPRINTF((BM64_DBG_COMMENT, "Bm64SendProcess !ERROR! returned FUNC:LibBM64_ExitPairingMode %d.\n", ret));
    }

    DPRINTF((BM64_DBG_FUNC_EXIT, "ExitPairingMode exit %d.\n", ret));
    return(ret);
}

int LibBM64_LinkLastDevice( void )
{
    ssize_t ret;
    unsigned char buf[BM64_WORK_BUFF_SIZE];

    DPRINTF((BM64_DBG_FUNC_ENTER, "LibBM64_LinkLastDevice enter.\n"));

#if 0
    //MMI_Action:link last device
    buf[0] = 0xAA;
    buf[1] = 0x00;
    buf[2] = 0x03;
    buf[3] = 0x02;
    buf[4] = 0x00;
    buf[5] = 0x6C;
    buf[6] = Bm64SumCalc(buf, BM64_MMI_ACTION_SIZE);
#endif
#if 1
    //MMI_Action:link last device
    buf[0] = 0xAA;
    buf[1] = 0x00;
    buf[2] = 0x03;
    buf[3] = 0x17;
    buf[4] = 0x00;
    buf[5] = Bm64SumCalc(buf, 6);
#endif

//    ret = Bm64SendProcess(buf, BM64_MMI_ACTION_SIZE);
    ret = Bm64SendProcess(buf, 6);
    if (ret < 0)
    {
        DPRINTF((BM64_DBG_COMMENT, "Bm64SendProcess !ERROR! returned FUNC:LibBM64_LinkLastDevice %d.\n", ret));
    }

    DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_LinkLastDevice exit %d.\n", ret));
    return(ret);
}

int LibBM64_DisconnectAllLink( void )
{
    ssize_t ret;
    unsigned char buf[BM64_WORK_BUFF_SIZE];

    DPRINTF((BM64_DBG_FUNC_ENTER, "LibBM64_DisconnectAllLink enter.\n"));

    #if 0
    //MMI_Action:disconnect all link
    buf[0] = 0xAA;
    buf[1] = 0x00;
    buf[2] = 0x03;
    buf[3] = 0x02;
    buf[4] = 0x00;
    buf[5] = 0x6D;
    buf[6] = Bm64SumCalc(buf, BM64_MMI_ACTION_SIZE);
    #endif

    #if 1
    //MMI_Action:disconnect all link
    buf[0] = 0xAA;
    buf[1] = 0x00;
    buf[2] = 0x03;
    buf[3] = 0x18;
    buf[4] = 0x3f;
    buf[5] = Bm64SumCalc(buf, 6);
    #endif

    //ret = Bm64SendProcess(buf, BM64_MMI_ACTION_SIZE);
    ret = Bm64SendProcess(buf, 6);
    if (ret < 0)
    {
        DPRINTF((BM64_DBG_COMMENT, "Bm64SendProcess !ERROR! returned FUNC:LibBM64_DisconnectAllLink %d.\n", ret));
    }

    DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_DisconnectAllLink exit %d.\n", ret));
    return(ret);
}

int LibBM64_SendUserCommand( unsigned char *pBuffer, unsigned int Length )
{
    ssize_t ret;
    unsigned int pktLen;
    unsigned char buf[BM64_WORK_BUFF_SIZE];

    DPRINTF((BM64_DBG_FUNC_ENTER, "--> LibBM64_SendUserCommand enter.\n"));

    if (LibInit == 0)
    {
        DPRINTF((BM64_DBG_ERROR, "Library is not initialized.\n"));
        DPRINTF((BM64_DBG_FUNC_EXIT, "LibBM64_SendUserCommand exit %d.\n", BM64_STATUS_NOT_INITIALIZE));
        return(BM64_STATUS_NOT_INITIALIZE);
    }

    pktLen = Length + BM64_PACKET_APPENDIX;
    DPRINTF((BM64_DBG_COMMENT, "packet length %d.\n", pktLen));

    if (BM64_WORK_BUFF_SIZE < pktLen)
    {
        DPRINTF((BM64_DBG_ERROR, "command length too big.\n"));
        ret = BM64_STATUS_PARAM_ERROR;
        goto EXIT_FUNC;
    }

    DPRINTF((BM64_DBG_COMMAND, "Command:x%02x.\n", pBuffer[0]));

    buf[0] = BM64_SYNC_WORD;
    Bm64PayloadLengthSet(buf, Length);
    memcpy(&buf[3], pBuffer, Length);
    buf[pktLen - 1] = Bm64SumCalc(buf, pktLen);

    ret = Bm64SendProcess(buf, pktLen);
    if (ret < 0)
    {
        DPRINTF((BM64_DBG_COMMENT, "Bm64SendProcess !ERROR! returned FUNC:LibBM64_SendUserCommand %d.\n", ret));
    }

EXIT_FUNC:
    DPRINTF((BM64_DBG_FUNC_EXIT, "<-- LibBM64_SendUserCommand exit %d.\n", ret));
    return(ret);
}
