#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "TCPIPReceiver.h"
#include "ControllerIF.h"
#include "UtilityforDebug.h"

static uint32_t CreateAMCF1Command(uint8_t* ref, uint32_t refSize, const char* command, const uint8_t* value);
static int32_t TcpIpSend(int32_t socket, const uint8_t* value, uint32_t size);

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  SendControlCommand			                                            */
/*                                                                          */
/* 2.Description                                                            */
/*  コマンド送信処理						                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*                                                                          */
/* 4.Return                                                                 */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t SendControlCommand(int32_t socket, const uint8_t* command, const uint8_t* value)
{
	uint32_t size = 0;
	uint8_t sendBuffer[LEN_COMMAND] = {};
	
	size = CreateAMCF1Command(&sendBuffer[0], LEN_COMMAND, command, value);
	if(size > 0)
	{
																DBGPrintf("Send(%d) %x-%x\n", size, sendBuffer[0],sendBuffer[1]);
		return TcpIpSend(socket,sendBuffer, size);
	}
	else
	{
		ERRPrintf("Send:Error\n");
		return -1;
	}
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CreateAMCF1Command		                                                */
/*                                                                          */
/* 2.Description                                                            */
/*  AMCF1コマンドを作成送信する　			     		                    */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static uint32_t CreateAMCF1Command(uint8_t* ref, uint32_t refSize, const char* command, const uint8_t* value)
{
	uint32_t size = 0;
	
	// [注意] 2byte以上のデータはビックエンディアンで取り扱うこと
	static uint8_t sendAlive[1]			= {	0xFF															};
	static uint8_t sendGainPos[5]		= {	0x91,	0x03,	0x01,	0x00,	0x00							};
	static uint8_t sendMuteMic[5]		= {	0x98,	0x03,	0x00,	0x00,	0x00							};
	static uint8_t sendMuteSpk[5]		= {	0x98,	0x03,	0x01,	0x00,	0x00							};
	static uint8_t sendBluetooth[4]		= {	0xF5,	0x02,	0x00,	0x00        							};
	static uint8_t sendPreset[4]		= {	0xF1,	0x02,	0x00,	0x00									};
	static uint8_t sendStandby[4]		= {	0xF3,	0x02,	0x00,	0x00									};
	static uint8_t sendStartup[3]		= {	0xDF,	0x01,	0x01											};
	static uint8_t sendFocus[8] 		= {	0xD0,	0x06,	0xA0,	0x00,	0x00,	0x00,	0x00,	0x00	};
	static uint8_t sendFocusSet[7] 		= {	0xA0,	0x05,	0x00,	0x00,	0x00,	0x00,	0x00			};
	static uint8_t sendLogin[3]			= {	0x80,	0x01,	0x00											};
	static uint8_t sendLogoff[2]		= {	0x81,	0x00													};
	static uint8_t sendAuto[4]			= {	0xF2,	0x02,	0x00,	0x00									};
	static uint8_t sendFocusStts[6] 	= {	0xF2,	0x04,	0x01,	0x00,	0x00,	0x00					};

	memset(ref, 0, refSize);
	
	if(strcmp(command, SEND_KEY_ALIVE) == 0) {
		memcpy(ref, sendAlive, sizeof(sendAlive));		    					size = sizeof(sendAlive);
	} else if(strcmp(command, SEND_KEY_GAIN) == 0) {
		memcpy(ref, sendGainPos, sizeof(sendGainPos));  	ref[4] = *value;	size = sizeof(sendGainPos);
	} else if(strcmp(command, SEND_KEY_MUTE_MIC) == 0) {
		memcpy(ref, sendMuteMic, sizeof(sendMuteMic));	    ref[4] = *value;	size = sizeof(sendMuteMic);
	} else if(strcmp(command, SEND_KEY_MUTE_SPK) == 0) {
		memcpy(ref, sendMuteSpk, sizeof(sendMuteSpk));  	ref[4] = *value;	size = sizeof(sendMuteSpk);
	} else if(strcmp(command, SEND_KEY_BLUETOOTH) == 0) {
		memcpy(ref, sendBluetooth, sizeof(sendBluetooth));	ref[3] = *value;	size = sizeof(sendBluetooth);
	} else if(strcmp(command, SEND_KEY_PRESET) == 0) {
		memcpy(ref, sendPreset, sizeof(sendPreset));	    ref[3] = *value;	size = sizeof(sendPreset);
	} else if(strcmp(command, SEND_KEY_STANDBY) == 0) {
		memcpy(ref, sendStandby, sizeof(sendStandby));	    ref[3] = *value;	size = sizeof(sendStandby);
	} else if(strcmp(command, SEND_KEY_STARTUP) == 0) {
		memcpy(ref, sendStartup, sizeof(sendStartup));	    					size = sizeof(sendStartup);
	} else if(strcmp(command, SEND_KEY_FOCUS) == 0) {
		memcpy(ref, sendFocus, sizeof(sendFocus));
		// X座標(ビックエンディアン)
		ref[3] = value[1];
		ref[4] = value[0];
		// Y座標(ビックエンディアン)
		ref[5] = value[3];
		ref[6] = value[2];
		// 単位系
		ref[7] = value[4];

		size = sizeof(sendFocus);
	} else if(strcmp(command, SEND_KEY_FOCUS_SET) == 0) {
		memcpy(ref, sendFocusSet, sizeof(sendFocusSet));
		// auto or manual
		ref[2] = value[0];

		// auto
		if(value[0] == 0x00) {
			// 初期値のまま
		}
		// manual
		else {
			// direction
			ref[3] = value[1];
			// distance(ビックエンディアン)
			ref[4] = value[5];
			ref[5] = value[4];
			// 単位系
			ref[6] = value[2];
		}
		size = sizeof(sendFocusSet);
	} else if(strcmp(command, SEND_KEY_LOGIN) == 0) {
		memcpy(ref, sendLogin, sizeof(sendLogin));		ref[2] = *value;	size = sizeof(sendLogin);
	} else if(strcmp(command, SEND_KEY_LOGOFF) == 0) {
		memcpy(ref, sendLogoff, sizeof(sendLogoff));						size = sizeof(sendLogoff);
	} else if(strcmp(command, SEND_KEY_AUTO) == 0) {
		memcpy(ref, sendAuto, sizeof(sendAuto));		ref[3] = *value;	size = sizeof(sendAuto);
	} else if(strcmp(command, SEND_KEY_FOCUS_STTS) == 0) {
		memcpy(ref, sendFocusStts, sizeof(sendFocusStts));

		ref[3] = value[0];
		ref[4] = value[1];
		ref[5] = value[2];
		
		size = sizeof(sendFocusStts);
	} else {
																DBGPrintf("CreateAMCF1Command:command(%10s) not found.\n", command);
	}
	return size;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  TcpIpSend				                                                */
/*                                                                          */
/* 2.Description                                                            */
/*  ソケット送信処理			　			                              	*/
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t TcpIpSend(int32_t socket, const uint8_t* value, uint32_t size)
{
	int32_t status = 0;
	
//	status = write(socket, value, size);
	status = send(socket, value, size, 0);
	if(status  < 0)
	{
		ERRPrintf("send failed:%d\n", status);
		return -1;
	}

	return 0;
}
