/*--------------------------------------------------------------------------*/
/*																			*/
/*	LibCF1CommCtrl.c (Rev 1.00)												*/
/*																			*/
/*	1.概要																	*/
/*		AM-CF1外部制御プロトコル											*/
/*																			*/
/*	2.改版履歴																*/
/*																			*/
/*																			*/
/*--------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/file.h>
#include <math.h>
#include "ControllerIF.h"
#include "TCPIPReceiver.h"
#include "CtimerLib.h"
#include "LibCF1CommCtrl.h"
#include "UtilityforDebug.h"
#include "CommStatusConverter.h"


// 受信コマンド
#define TCP_REQ_GAIN			0x91
#define TCP_REQ_MUTE			0x98
#define TCP_INF_STATUS			0xF0
#define TCP_INF_FOCUS_SET		0xA0
#define TCP_REQ_PRESET			0xF1
#define TCP_INF_STATUS_SET		0xF2
#define TCP_REQ_STANDBY			0xF3
#define TCP_REQ_LOGIN			0x80
#define TCP_REQ_LOGOFF			0x81
#define TCP_REQ_BLUETOOTH		0xF5

// ChannelAttribute
#define CH_ATTR_MIC_IN		0x00
#define CH_ATTR_SPK_OUT		0x01

// 受信コマンド(ステータスリクエスト)
#define GAIN_REQUEST 		0x11
#define MUTE_REQUEST 		0x18
#define FOCUS_SET_REQUEST 	0x20
#define FOCUS_POS_REQUEST 	0x50
#define PRESET_REQUEST 		0x71
#define STANDBY_REQUEST 	0x72
#define BLUETOOTH_REQUEST 	0x74

// 受信コマンド(ステイタス通知 設定)
#define AUTO_STATUS_SETTING		0x00
#define FOCUS_STATUS_SETTING	0x01

#define RES_OFF				0	// 応答無
#define RES_ON				1	// 応答有
#define AUTO_STATUS_OFF		0	// 自動応答有
#define AUTO_STATUS_ON		1	// 自動応答有
#define FOCUS_AUTO			0	// ビームステアリングのAutoモード
#define FOCUS_MANUAL		1	// ビームステアリングのManualモード

enum BLUETOOTH_PARAM {
	BLUETOOTH_PARAM_OFF = 0,	// Bluetooth通信の終了
	BLUETOOTH_PARAM_ON,			// Bluetooth通信の開始
	BLUETOOTH_PARAM_MAX,
};

#define FOCUS_X_REQUEST		0x00	// FOCUS_REQUESTの詳細パラメータ：X座標
#define FOCUS_Y_REQUEST		0x01	// FOCUS_REQUESTの詳細パラメータ：Y座標

#define ALIVE_TIMER_TIME 	10000 	// ALIVE送信間隔(msec)：10秒

// 制御イベント取得用
#define KEY_GAIN 			SEND_KEY_GAIN
#define KEY_MUTE_MIC 		SEND_KEY_MUTE_MIC
#define KEY_MUTE_SPK 		SEND_KEY_MUTE_SPK
#define KEY_PRESET 			SEND_KEY_PRESET
#define KEY_FOCUS_SET		SEND_KEY_FOCUS_SET
#define KEY_STANDBY			SEND_KEY_STANDBY
#define KEY_BLUETOOTH		SEND_KEY_BLUETOOTH

#define FOCUS_TIME_INIT		(0)		// 送信間隔(初期値)
#define FOCUS_TIME_MIN		(0)		// 即時応答(機器情報通知毎に送信する)
#define FOCUS_TIME_MAX		(10)	// 100msec単位で設定可能とする
#define FOCUS_TIME_UNIT		(100)	// 100msecに変換する
#define FOCUS_NO_TIME		FOCUS_TIME_MIN
#define FOCUS_DIR_INIT		(0)		// manualの角度設定(初期値)
#define FOCUS_DIR_MIN		(-90)	// manualの角度設定(最小)
#define FOCUS_DIR_MAX		(90)	// manualの角度設定(最大)
#define FOCUS_DIS_INIT		(1.0)	// manualの距離設定(初期値)
#define FOCUS_DIS_MIN		(0.0)	// manualの距離設定(最小)
#define FOCUS_DIS_MAX		(6.0)	// manualの距離設定(最大)
#define FOCUS_UNIT_INIT		(0)		// manualの単位設定(初期値)

// ログイン情報
#define LOGIN_PATH_MAX		1024	// ファイルパス長
#define LOGIN_FILE_LINE		256		// 1行の最大バイト数
#define LOGIN				1		// ログイン状態：ログイン
#define LOGOFF				0		// ログイン状態：ログオフ
#define USERNAME_MAX		16		// ユーザ名：最大16バイト
#define USERPASS_MAX		16		// ユーザパス：最大16バイト
#define PADDING_CHAR		0x00	//　通信IFのユーザ名・パスの16バイト未満の時のパディングコード
#define USER_MARK			"USER"
#define USER_MARK_SIZE		4
#define PASS_MARK			"PASS"
#define PASS_MARK_SIZE		4
#define SKIP_LOGIN_ENABLE	0		// ログイン機能を無効にする
#define SKIP_LOGIN_DISABLE	1		// ログイン機能を有効にする

#define BT_STATUS_OFF		0		// BLUETOOTHステータス: OFF
#define BT_STATUS_PAIRING	1		// BLUETOOTHステータス: ペアリング登録中
#define BT_STATUS_CONNECTED	2		// BLUETOOTHステータス: 接続中

// 応答有無テーブル要素番号
enum ResMessage {
	ResMessageGain = 0,
	ResMessageMuteMic,
	ResMessageMuteSpk,
	ResMessageBluetooth,
	ResMessagePreset,
	ResMessageFocusSet,
	ResMessageStandby,
	ResMessageNumMax,
};

void main(void); //削除する

static int8_t	SetGain(uint8_t value);
static int8_t	SetMuteMic(uint8_t value);
static int8_t	SetMuteSpk(uint8_t value);
static int8_t	SetBluetooth(uint8_t value);
static int8_t	SetPreset(uint8_t value);
static int8_t	SetStandby(uint8_t value);
static int8_t	SetFocusSetting(uint8_t mode, int8_t dis, uint16_t dir, uint8_t unit);
static int32_t	SetAutoResponse(uint32_t key_size1, uint32_t key_size2, char key[key_size1][key_size2], stCommStatus* statusTmp);
static int32_t	SendFocusStatus(void);
static void		UpdateFocusTimer(void);
static int32_t	Send(const char* command, const uint8_t* value);
static uint8_t* GetCommStatus(stCommStatus* target, const char* key);
static int32_t	Checklogin(const uint8_t* filePath, const uint8_t* name, const uint8_t* pass);
static uint8_t*	GetLoginStr(const uint8_t* readLine, uint32_t readSize, const uint8_t* markStr, uint32_t markSize);
static int32_t	CheckloginStr(const uint8_t* checkName, uint32_t checkMaxSize, uint8_t terminate, const uint8_t* destName);
static uint8_t* MakeFocusData(stCommStatus* target, const char* key, uint8_t unit);
static uint8_t* MakeFocusSetData(stCommStatus* target, const char* key);

// LibCF1CommCtrl_Controlで受信したコマンドごとの処理関数
static int32_t	CtrlReqGain(uint8_t* receivedCommand);
static int32_t	CtrlReqMute(uint8_t* receivedCommand);
static int32_t	CtrlReqBluetooth(uint8_t* receivedCommand);
static int32_t	CtrlReqPreset(uint8_t* receivedCommand);
static int32_t	CtrlReqStandby(uint8_t* receivedCommand);
static int32_t	CtrlInfStatus(uint8_t* receivedCommand);
static int32_t	CtrlInfAutoStatus(uint8_t* receivedCommand);
static int32_t	CtrlLogin(uint8_t* receivedCommand);
static int32_t	CtrlLogoff(uint8_t* receivedCommand);
static int32_t	CtrlInfFocusSetting(uint8_t* receivedCommand);
static int32_t	CtrlInfFocusStatus(uint8_t* receivedCommand);
static int32_t	CtrlInfStatusSetting(uint8_t* receivedCommand);

// 通信切断時に呼び出されるコールバック関数
static void DisconnectedCallback(void);

// エンディアンの変換関数
static int32_t ConvertEndian(uint8_t* src, uint8_t* dst, uint32_t size);
// 単位変換
static float ConvertInchToM(uint16_t x);
static float ConvertCmToM(uint16_t x);
static int32_t ConvertMToCm(float x);
static int32_t ConvertMToInch(float x);
// 四捨五入
static int32_t round_f(float x);

static uint8_t	_resMessage[ResMessageNumMax] = {0};	// 応答有無テーブル
static char		_aliveTimer[] = "AliveTimer";			// ALIVETIMER名
static char		_focusTimer[] = "FocusTimer";			// FOCUSTIMER名
static uint8_t	_LoginOk = 1;
static uint8_t	_LoginNg = 0;
static struct {
	uint8_t autoResponse;
	uint8_t autoResponseUnit;
	uint8_t interval;
} _focusStatus = { AUTO_STATUS_OFF, FOCUS_UNIT_INCH, FOCUS_TIME_INIT };

static uint8_t _autoResponse = AUTO_STATUS_OFF;			// 自動応答設定
static stCommStatus _controlData;						// 制御指示データ
static stCommStatus _statusData;						// 状態データ

stCommStatusFlg _controlDataFlg;						// 制御指示データ受信フラグ

static uint8_t	_loginInfoPath[LOGIN_PATH_MAX] = {0};	// ログイン情報のファイルパス
static uint8_t	_loginState = LOGOFF;					// ログイン状態(初期はログオフ)
static uint8_t	_skipLogin;						// パスワード認証の有効 or 無効

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibCF1CommCtrl_Init                             	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  ライブラリ初期化						                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t LibCF1CommCtrl_Init(const uint8_t* loginInfoPath, uint32_t pathSize, uint32_t isSkipLogin)
{
	int32_t ret = RET_OK;

	memset(&_controlData, 0, sizeof(_controlData));
	memset(&_controlDataFlg, 0, sizeof(_controlDataFlg));
	memset(&_statusData, 0, sizeof(_statusData));
	ret = TCPIPReceiver_Init(DisconnectedCallback); // 受信スレッドを起動する
	if (ret == 0) {
		ret = LibCTIMER_Init();
		if (ret == CTIMER_STATUS_SUCCESS) {
			ret = LibCTIMER_setTimer(_aliveTimer, ALIVE_TIMER_TIME);
		}
	}

	_controlData.gain = GAIN_INVALID;
	_controlData.muteMic = MUTE_INVALID;
	_controlData.muteSpk = MUTE_INVALID;
	_controlData.bluetoothMode = BLUETOOTH_INVALID;
	_controlData.standby = STANDBY_INVALID;
	_controlData.preset = PRESET_INVALID;
	_controlData.focus_x = FOCUS_X_INVALID;
	_controlData.focus_y = FOCUS_Y_INVALID;
	_controlData.focus_mode = FOCUS_AUTO;
	_controlData.focus_direction = FOCUS_DIR_INIT;
	_controlData.focus_distance = FOCUS_DIS_INIT;
	_controlData.focus_unit = FOCUS_UNIT_INIT;
	_controlDataFlg.gainUpdated = 0;
	_controlDataFlg.muteMicUpdated = 0;
	_controlDataFlg.muteSpkUpdated = 0;
	_controlDataFlg.bluetoothUpdated = 0;
	_controlDataFlg.standbyUpdated = 0;
	_controlDataFlg.presetUpdated = 0;
	_controlDataFlg.focusSetUpdated = 0;
	_statusData.gain = GAIN_INVALID;
	_statusData.muteMic = MUTE_INVALID;
	_statusData.muteSpk = MUTE_INVALID;
	_statusData.bluetoothMode = BLUETOOTH_INVALID;
	_statusData.standby = STANDBY_INVALID;
	_statusData.preset = PRESET_INVALID;
	_statusData.focus_x = FOCUS_X_INVALID;
	_statusData.focus_y = FOCUS_Y_INVALID;
	_statusData.focus_mode = FOCUS_AUTO;
	_statusData.focus_direction = FOCUS_DIR_INIT;
	_statusData.focus_distance = FOCUS_DIS_INIT;
	_statusData.focus_unit = FOCUS_UNIT_INIT;

	memset(_loginInfoPath, 0x00, sizeof(_loginInfoPath));
	memcpy(&_loginInfoPath[0], loginInfoPath, pathSize);
	_loginState = LOGOFF;

	if (isSkipLogin == TRUE) {
		_skipLogin = SKIP_LOGIN_ENABLE;
	} else {
		_skipLogin = SKIP_LOGIN_DISABLE;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibCF1CommCtrl_Deinit                             	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  ライブラリ終了  						                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t LibCF1CommCtrl_Deinit(void)
{
	int32_t ret = RET_OK;

	ret |= TCPIPReceiver_Deinit(); // 受信スレッドを終了する
	ret |= LibCTIMER_Deinit();

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibCF1CommCtrl_Control                             	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  受信コマンド取得処理（定周期呼び出し）	                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t LibCF1CommCtrl_Control( void )
{
	int32_t ret = RET_OK;
	uint32_t i = 0, j = 0;
	int32_t request = 0;
	stReceiveBuffer buf;
	struct {
		uint8_t command;					// 受信コマンド
		int32_t cmdIndex;					// インデックス(-1: コマンド受信なし)
		int32_t (*func)(uint8_t*);			// 処理関数ポインタ
	} cmdTable[] =
	{
		{ TCP_REQ_LOGIN,		-1,	CtrlLogin				},
		{ TCP_REQ_LOGOFF,		-1,	CtrlLogoff				},
		{ TCP_REQ_GAIN,			-1,	CtrlReqGain				},
		{ TCP_REQ_MUTE,			-1,	CtrlReqMute				},
		{ TCP_REQ_BLUETOOTH,	-1,	CtrlReqBluetooth		},
		{ TCP_REQ_PRESET,		-1,	CtrlReqPreset			},
		{ TCP_REQ_STANDBY,		-1,	CtrlReqStandby			},
		{ TCP_INF_STATUS,		-1,	CtrlInfStatus			},
		{ TCP_INF_STATUS_SET,	-1,	CtrlInfStatusSetting	},
		{ TCP_INF_FOCUS_SET,	-1,	CtrlInfFocusSetting		},
	};

	memset(&buf, 0x00, sizeof(buf));

	// 受信したコマンドを取得
	GetReceiveCommand(&buf);

	if (buf.num > 0) {
#if 0
		// コマンドごとの最新の受信コマンドインデックスを探す
		for (i = 0; i < buf.num; i++)
		{
			for (j = 0; j < (sizeof(cmdTable) / sizeof(cmdTable[0])); j++)
			{
				if (buf.receivedCommand[i][0] == cmdTable[j].command) {
					cmdTable[j].cmdIndex = i;
				}
			}
		}
#else
		// ※現状、コントローラに応答を返すまで次のコマンドが来ない仕様なので
		// 　最新のコマンドのみ処理を行う

		// 最新のコマンドのコマンド種別判定。
		for (j = 0; j < (sizeof(cmdTable) / sizeof(cmdTable[0])); j++)
		{
			if (buf.receivedCommand[buf.num - 1][0] == cmdTable[j].command) {
				cmdTable[j].cmdIndex = buf.num - 1;
			}
		}
#endif

		// ログオフ中でパスワード認証のスキップが有効であれば、ログインとログオフ以外のコマンドでログイン状態をログイン中にする
		if (_loginState == LOGOFF) {
			if (_skipLogin == SKIP_LOGIN_ENABLE) {
				for (j = 0; j < (sizeof(cmdTable) / sizeof(cmdTable[0])); j++)
				{
					if ((buf.receivedCommand[buf.num - 1][0] == cmdTable[j].command)
					&& ((cmdTable[j].command != TCP_REQ_LOGIN) && (cmdTable[j].command != TCP_REQ_LOGOFF))) {
						_loginState = LOGIN;
						break;
					}
				}
			}
		}

		// ログオフ中はログインコマンド以外は無視する
		if (_loginState == LOGOFF) {
			for (j = 0; j < (sizeof(cmdTable) / sizeof(cmdTable[0])); j++)
			{
				if ((buf.receivedCommand[buf.num - 1][0] == cmdTable[j].command)
				 && (cmdTable[j].command != TCP_REQ_LOGIN)) {
					cmdTable[j].cmdIndex = -1;
					// ログオフ中がコントローラで判断できるようにログインのNACK応答を送信する
					Send(SEND_KEY_LOGIN, &_LoginNg);

					// コマンドは現状、応答を返すまで次のコマンドが来ない仕様なので
					// 1つのコマンドを処理したらループから抜ける。
					break;
				}
			}
		}

		// 受信したコマンドごとの処理関数を呼び出す
		for (j = 0; j < (sizeof(cmdTable) / sizeof(cmdTable[0])); j++)
		{
			if (cmdTable[j].cmdIndex != -1) {
				ret = cmdTable[j].func(buf.receivedCommand[cmdTable[j].cmdIndex]);
			}
		}
	}

	// タイムアウトしている場合はALIVEを送信する
	if (LibCTIMER_getTimeout(_aliveTimer) == 1) {
																	DBGPrintf("%s:TimeOutSendTimer Timeout\n", __FUNCTION__);
		if (GetSocket() != SOCKET_NULL) {
			if (Send(SEND_KEY_ALIVE, NULL) == -1) {
				ret = RET_NG;
			}
		}
		else {
			// 接続されていない場合はタイマーを更新する
			LibCTIMER_setTimer(_aliveTimer, ALIVE_TIMER_TIME);
			ret = RET_NG;
		}
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CtrlReqGain                 		            	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  チャンネルフェーダーゲイン設定コマンド受信時の処理                      */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	receivedCommand		受信バッファポインタ					*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t CtrlReqGain(uint8_t* receivedCommand)
{
	int32_t ret = 0;

	if (!receivedCommand) {
		ret = -1;
		return ret;
	}

	if (*GetCommStatus(&_statusData, KEY_STANDBY) == 1) {
		// スタンバイモードの場合は、ここで応答送信
		Send(KEY_GAIN, GetCommStatus(&_statusData, KEY_GAIN));
		return ret;
	}

	if (SetGain(receivedCommand[4]) == 0) {		// SetGain(): OK
		//															DBGPrintf("応答ON[%s]\n", KEY_GAIN);
		//_resMessage[ResMessageGain] = RES_ON; // 応答ON
		_controlDataFlg.gainUpdated = 1;
	}
	else {									// SetGain(): NG
	 // 不正な設定値の場合は、ここで応答送信
		Send(KEY_GAIN, GetCommStatus(&_statusData, KEY_GAIN));
		ret = -1;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CtrlReqMute                 		            	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  チャンネルミュートON/OFF設定コマンド受信時の処理                        */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	receivedCommand		受信バッファポインタ					*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t CtrlReqMute(uint8_t* receivedCommand)
{
	int32_t ret = 0;

	if (!receivedCommand) {
		ret = -1;
		return ret;
	}

	if (receivedCommand[2] == CH_ATTR_MIC_IN) {				// MIC MUTE
		if (*GetCommStatus(&_statusData, KEY_STANDBY) == 1) {
			// スタンバイモードの場合は、ここで応答送信
			Send(KEY_MUTE_MIC, GetCommStatus(&_statusData, KEY_MUTE_MIC));
			return ret;
		}

		if (SetMuteMic(receivedCommand[4]) == 0) {	// SetMuteMic(): OK
			_controlDataFlg.muteMicUpdated = 1;
		} else {								// SetMute(): NG
			// 不正な設定値の場合は、ここで応答送信
			Send(KEY_MUTE_MIC, GetCommStatus(&_statusData, KEY_MUTE_MIC));
			ret = -1;
		}
	}
	else if (receivedCommand[2] == CH_ATTR_SPK_OUT) {		// SPEAKER MUTE
		if (*GetCommStatus(&_statusData, KEY_STANDBY) == 1) {
			// スタンバイモードの場合は、ここで応答送信
			Send(KEY_MUTE_SPK, GetCommStatus(&_statusData, KEY_MUTE_SPK));
			return ret;
		}

		if (SetMuteSpk(receivedCommand[4]) == 0) {		// SetMuteSpk(): OK
			_controlDataFlg.muteSpkUpdated = 1;
		}
		else {								// SetMute(): NG
											// 不正な設定値の場合は、ここで応答送信
			Send(KEY_MUTE_SPK, GetCommStatus(&_statusData, KEY_MUTE_SPK));
			ret = -1;
		}
	}
	else {
		// 不正な設定値の場合は、ここで応答送信
		Send(KEY_MUTE_MIC, GetCommStatus(&_statusData, KEY_MUTE_MIC));
		ret = -1;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CtrlReqBluetooth              		            	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  BluetoothモードON/OFF設定コマンド受信時の処理                        	*/
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	receivedCommand		受信バッファポインタ					*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t CtrlReqBluetooth(uint8_t* receivedCommand)
{
	int32_t ret = 0;

	if (!receivedCommand) {
		ret = -1;
		return ret;
	}

	// スタンバイモードの場合は、ここで応答送信
	if (*GetCommStatus(&_statusData, KEY_STANDBY) == 1) {
		Send(KEY_BLUETOOTH, GetCommStatus(&_statusData, KEY_BLUETOOTH));
		return ret;
	}

	if (SetBluetooth(receivedCommand[3]) == RET_OK) {
																	DBGPrintf("応答ON[%s]\n", KEY_BLUETOOTH);
		_controlDataFlg.bluetoothUpdated = 1;
	}
	// 不正な設定値の場合は、ここで応答送信
	else {
		Send(KEY_BLUETOOTH, GetCommStatus(&_statusData, KEY_BLUETOOTH));
		ret = -1;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CtrlReqPreset                 		            	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  プリセットメモリ設定コマンド受信時の処理                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	receivedCommand		受信バッファポインタ					*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t CtrlReqPreset(uint8_t* receivedCommand)
{
	int32_t ret = 0;

	if (*GetCommStatus(&_statusData, KEY_STANDBY) == 1) {
		// スタンバイモードの場合は、ここで応答送信
		Send(KEY_PRESET, GetCommStatus(&_statusData, KEY_PRESET));
		return ret;
	}

	if (!receivedCommand) {
		ret = -1;
		return ret;
	}

	if (SetPreset(receivedCommand[3]) == 0) {	// SetPreset(): OK
		//															DBGPrintf("応答ON[%s]\n", KEY_PRESET);
		//_resMessage[ResMessagePreset] = RES_ON; // 応答ON
		_controlDataFlg.presetUpdated = 1;
	} else {									// SetPreset(): NG
		// 不正な設定値の場合は、ここで応答送信
		Send(KEY_PRESET, GetCommStatus(&_statusData, KEY_PRESET));
		ret = -1;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CtrlReqStandby                 		            	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  スタンバイモード切替設定コマンド受信時の処理                            */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	receivedCommand		受信バッファポインタ					*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t CtrlReqStandby(uint8_t* receivedCommand)
{
	int32_t ret = 0;

	if (!receivedCommand) {
		ret = -1;
		return ret;
	}

	if (SetStandby(receivedCommand[3]) == 0) {	// SetStandby(): OK
		//															DBGPrintf("応答ON[%s]\n", KEY_STANDBY);
		//_resMessage[ResMessageStandby] = RES_ON; // 応答ON
		_controlDataFlg.standbyUpdated = 1;
	} else {									// SetStandby(): NG
		// 不正な設定値の場合は、ここで応答送信
		Send(KEY_STANDBY, GetCommStatus(&_statusData, KEY_STANDBY));
		ret = -1;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CtrlInfFocusSetting                 		       	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  ビームステアリング設定コマンド受信時の処理							    */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	receivedCommand		受信バッファポインタ					*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t CtrlInfFocusSetting(uint8_t* receivedCommand)
{
	int32_t ret = 0;
	uint8_t mode;
	int8_t dir;
	uint16_t dis;
	uint8_t unit;

	if (!receivedCommand) {
		ret = -1;
		return ret;
	}

	// スタンバイモードの場合は、ここでNG応答送信
	if (*GetCommStatus(&_statusData, KEY_STANDBY) == 1) {
		ret |= Send(KEY_FOCUS_SET, MakeFocusSetData(&_statusData, KEY_FOCUS_SET));
		return ret;
	}

	mode = receivedCommand[2];
	dir = (int8_t)receivedCommand[3];
	ConvertEndian(&receivedCommand[4], (uint8_t*)&dis, sizeof(uint16_t));
	unit = receivedCommand[6];

	if (SetFocusSetting(mode, dir, dis, unit) == RET_OK) {
		_controlDataFlg.focusSetUpdated = 1;
	}
	// 範囲外の値があれば、ここでNG応答送信
	else {
		ret |= Send(KEY_FOCUS_SET, MakeFocusSetData(&_statusData, KEY_FOCUS_SET));
	}


	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CtrlInfStatus                 		            	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  ステイタスリクエストコマンド受信時の処理                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	receivedCommand		受信バッファポインタ					*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t CtrlInfStatus(uint8_t* receivedCommand)
{
	int32_t ret = 0;

	if (!receivedCommand) {
		ret = -1;
		return ret;
	}

	// 応答送信
	switch (receivedCommand[2])
	{
	case GAIN_REQUEST:
		ret = Send(KEY_GAIN, GetCommStatus(&_statusData, KEY_GAIN));
		break;
	case MUTE_REQUEST:
		if (receivedCommand[3] == CH_ATTR_SPK_OUT) {
			ret = Send(KEY_MUTE_SPK, GetCommStatus(&_statusData, KEY_MUTE_SPK));
		}
		else {
			// デフォルトはMICにしておく
			ret = Send(KEY_MUTE_MIC, GetCommStatus(&_statusData, KEY_MUTE_MIC));
		}
		break;
	case BLUETOOTH_REQUEST:
		ret = Send(KEY_BLUETOOTH, GetCommStatus(&_statusData, KEY_BLUETOOTH));
		break;
	case PRESET_REQUEST:
		ret = Send(KEY_PRESET, GetCommStatus(&_statusData, KEY_PRESET));
		break;
	case STANDBY_REQUEST:
		ret = Send(KEY_STANDBY, GetCommStatus(&_statusData, KEY_STANDBY));
		break;
	case FOCUS_SET_REQUEST:
		ret = Send(KEY_FOCUS_SET, MakeFocusSetData(&_statusData, KEY_FOCUS_SET));
		break;
	case FOCUS_POS_REQUEST:
		if (receivedCommand[7] <= FOCUS_UNIT_CM) {
			ret = Send(SEND_KEY_FOCUS, MakeFocusData(&_statusData, SEND_KEY_FOCUS, receivedCommand[7]));
		}
		else {
			// 範囲外は全てcmとして処理する
			ret = Send(SEND_KEY_FOCUS, MakeFocusData(&_statusData, SEND_KEY_FOCUS, FOCUS_UNIT_CM));
		}
		break;
	default: break; // none
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CtrlInfStatusSetting               		          	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  ステイタス通知 設定コマンド受信時の処理	                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	receivedCommand		受信バッファポインタ					*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t CtrlInfStatusSetting(uint8_t* receivedCommand)
{
	int32_t ret = 0;

	if (!receivedCommand) {
		ret = -1;
		return ret;
	}

	// 分岐
	switch (receivedCommand[2])
	{
	case AUTO_STATUS_SETTING:
		CtrlInfAutoStatus(receivedCommand);
		break;
	case FOCUS_STATUS_SETTING:
		CtrlInfFocusStatus(receivedCommand);
		break;
	default: break; // none
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CtrlInfAutoStatus                 		          	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  自動ステイタス通知コマンド受信時の処理	                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	receivedCommand		受信バッファポインタ					*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t CtrlInfAutoStatus(uint8_t* receivedCommand)
{
	int32_t ret = 0;

	if (receivedCommand[3] >= AUTO_STATUS_ON) {
		_autoResponse = AUTO_STATUS_ON;
	} else {
		_autoResponse = AUTO_STATUS_OFF;
	}

	// 応答送信
	ret |= Send(SEND_KEY_AUTO, &_autoResponse);

	// ONの場合は現在値を通知しておく
	if (_autoResponse == 1) {
		ret |= Send(KEY_GAIN, GetCommStatus(&_statusData, KEY_GAIN));
		ret |= Send(KEY_MUTE_MIC, GetCommStatus(&_statusData, KEY_MUTE_MIC));
		ret |= Send(KEY_MUTE_SPK, GetCommStatus(&_statusData, KEY_MUTE_SPK));
		ret |= Send(KEY_BLUETOOTH, GetCommStatus(&_statusData, KEY_BLUETOOTH));
		// 自動ステイタス通知ではビームステアリングは通知しない。
		ret |= Send(KEY_PRESET, GetCommStatus(&_statusData, KEY_PRESET));
		ret |= Send(KEY_STANDBY, GetCommStatus(&_statusData, KEY_STANDBY));
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CtrlInfFocusStatus                 		       	            	        */
/*                                                                          */
/* 2.Description                                                            */
/*  ビームステアリング ステイタス通知設定コマンド受信時の処理			    */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	receivedCommand		受信バッファポインタ					*/
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t	0: OK, -1: NG(不正値)											*/
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t	CtrlInfFocusStatus(uint8_t* receivedCommand)
{
	int32_t ret = 0;

	static struct stFocusStatus {
		uint8_t interval;
		uint8_t unit;
		uint8_t response;
	} focusStatusRes;

	// 範囲外の値は丸め込んで設定する
	if (receivedCommand[3] > FOCUS_TIME_MAX) {
		_focusStatus.interval = FOCUS_TIME_MAX;
	} else {
		_focusStatus.interval = receivedCommand[3];
	}
	if (receivedCommand[4] > FOCUS_UNIT_CM) {
		_focusStatus.autoResponseUnit = FOCUS_UNIT_CM;
	} else {
		_focusStatus.autoResponseUnit = receivedCommand[4];
	}
	if (receivedCommand[5] > AUTO_STATUS_ON) {
		_focusStatus.autoResponse = AUTO_STATUS_ON;
	} else {
		_focusStatus.autoResponse = receivedCommand[5];
	}

	// 応答送信データの作成
	focusStatusRes.interval = _focusStatus.interval;
	focusStatusRes.unit = _focusStatus.autoResponseUnit;
	focusStatusRes.response = _focusStatus.autoResponse;

	// 応答送信(LibCF1CommCtrl内で閉じた処理であるため)
	ret = Send(SEND_KEY_FOCUS_STTS, (uint8_t*)&focusStatusRes);

	// 設定に応じてタイマーの設定・解除を行う
	UpdateFocusTimer();

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CtrlLogin                                                               */
/*                                                                          */
/* 2.Description                                                            */
/*  ログインコマンド受信時の処理	                                        */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	receivedCommand		受信バッファポインタ	                */
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t		0: OK, -1: NG(不正値)	                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t CtrlLogin(uint8_t* receivedCommand)
{
	int32_t ret = 0;
	int32_t matchResult = RET_NG;

	if (!receivedCommand) {
		ret = -1;
		return ret;
	}
	// ログオフ中はログイン情報と照合する
	// ログイン中のログインは不一致と同様の扱いとする
	if (_loginState == LOGOFF) {
		matchResult = Checklogin(_loginInfoPath, &receivedCommand[2], &receivedCommand[18]);
	}

	if (matchResult == RET_OK) {
		_loginState = LOGIN;

		if (Send(SEND_KEY_LOGIN, &_LoginOk) != 0) {
			ret = -1;
		}
	}
	else {
		if (Send(SEND_KEY_LOGIN, &_LoginNg) != 0) {
			ret = -1;
		}
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  Checklogin                                                              */
/*                                                                          */
/* 2.Description                                                            */
/*  ログイン情報との照合	                                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	filePath		照合するログイン情報のファイルパス	        */
/*  uint8_t*	name			照合するログイン名	                        */
/*  uint8_t*	pass			照合するパスワード	                        */
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t		RET_OK: 一致, RET_NG: 不一致	                            */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t Checklogin(const uint8_t* filePath, const uint8_t* name, const uint8_t* pass)
{
	uint8_t readline[LOGIN_FILE_LINE] = {};
	FILE* file = NULL;
	uint8_t* loginName = NULL;
	uint8_t* loginPass = NULL;
	int32_t matchNameResult = RET_NG;
	int32_t matchPassResult = RET_NG;
	int32_t result = RET_NG;
	int32_t lock = -1;

	// ログイン情報と取得する
	// 対象のログイン情報が取得できない場合は、不一致とする
	file = fopen(filePath, "r");
	if (file != NULL) {
		lock = flock(fileno(file), LOCK_SH | LOCK_NB);

		if (lock == -1) {
			fclose(file);
			return RET_NG;
		}
	}
	else {
		return RET_NG;
	}

	// ログイン情報を照合する
	// 前提条件：ログインファイルのユーザ情報の並びが USER行, PASS行 の順で記述されていること
	while(1) {
		// ログイン名の照合(n行目)
		memset(readline, PADDING_CHAR, LOGIN_FILE_LINE);
		if (fgets(readline, LOGIN_FILE_LINE, file) == NULL) {
			// ファイル終端のため、処理を中断する
			break;
		}
		loginName = GetLoginStr(readline, LOGIN_FILE_LINE, USER_MARK, USER_MARK_SIZE);
		matchNameResult = CheckloginStr(name, USERNAME_MAX, PADDING_CHAR, loginName);

		// ログイン名の照合(n+1行目)
		memset(readline, PADDING_CHAR, LOGIN_FILE_LINE);
		if (fgets(readline, LOGIN_FILE_LINE, file) == NULL) {
			// ファイル終端のため、処理を中断する
			break;
		}
		loginPass = GetLoginStr(readline, LOGIN_FILE_LINE, PASS_MARK, PASS_MARK_SIZE);
		matchPassResult = CheckloginStr(pass, USERPASS_MAX, PADDING_CHAR, loginPass);

		// ログイン名とパスワードが一致すれば、以降の処理は不要となるため処理を中断する
		if ((matchNameResult == RET_OK) && (matchPassResult == RET_OK)) {
			result = RET_OK;
			break;
		}
	}

	flock(fileno(file), LOCK_UN);
	fclose(file);

	return result;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  GetLoginStr                                                             */
/*                                                                          */
/* 2.Description                                                            */
/*  ログイン文字列を取得する	                                            */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	readLine		ログイン文字列の読み取り先	                */
/*  uint32_t	readSize		ログイン文字列の読み取り最大サイズ	        */
/*  uint8_t*	markStr			識別文字	                                */
/*  uint32_t	markSize		識別文字数	                                */
/*                                                                          */
/* 4.Return                                                                 */
/*  uint8_t*	ログイン文字列の先頭アドレス	                            */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static uint8_t* GetLoginStr(const uint8_t* readLine, uint32_t readSize, const uint8_t* markStr, uint32_t markSize)
{
	uint8_t* delimiter = NULL;
	uint8_t* loginStr = NULL;

	if ((readLine == NULL) || (markStr == NULL)) {
		return NULL;
	}
	if (memcmp(&readLine[0], markStr, markSize) == 0)
	{
		delimiter = ((uint8_t*)memchr(&readLine[0], ',', readSize));
		// 区切り文字が目的の文字列の先頭アドレスになる
		if (delimiter != NULL) {
			loginStr = delimiter + 1;
		}
	}

	return loginStr;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CheckloginStr                                                           */
/*                                                                          */
/* 2.Description                                                            */
/*  ログイン文字列の照合	                                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	checkName		照合する文字列	                            */
/*  uint32_t	checkMaxSize	照合する文字列の最大サイズ	                */
/*  uint32_t	terminate		照合する文字列の終端記号	                */
/*  uint8_t*	destName		ログイン情報の文字列	                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t		RET_OK: 一致, RET_NG: 不一致	                            */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t CheckloginStr(const uint8_t* checkName, uint32_t checkMaxSize, uint8_t terminate, const uint8_t* destName)
{
	int32_t matchResult = RET_NG;
	uint8_t seek = 0;
	uint8_t matchCount = 0;
	uint8_t matchCountMax = 0;
	uint8_t isNoMatch = FALSE;

	// ログイン情報の文字列がなしの場合も許容する
	// 理由：
	//  コントローラーのログインはWebブラウザのログイン情報を参照するだけである。
	//  そのため、ログイン情報が文字列なしになっていればそれに従って照合する。

	if ((checkName != NULL) && (destName != NULL)) {

		// ログイン情報の文字数を取得する
		for(seek = 0; (seek < checkMaxSize); seek++) {
			// 制御コードを対象から外してカウントする
			if (((0x20 <= destName[seek]) && (destName[seek] <= 0x7E)) || (0x80 <= destName[seek])) {
				matchCountMax++;
			}
			else {
				break;
			}
		}

		// 文字列を照合する
		for(seek = 0; ((seek < checkMaxSize) && (checkName[seek] != terminate)); seek++) {
			if (checkName[seek] == destName[seek]) {
				matchCount++;
			}
			// 一致しない文字が1つでもあれば不一致とする
			else if (((0x20 <= checkName[seek]) && (checkName[seek] <= 0x7E)) || (0x80 <= checkName[seek])) {
				isNoMatch = TRUE;
			}
		}

		// ログイン情報の文字数と一致した文字数が同じであれあば、一致したとする
		if (isNoMatch == FALSE) {
			if (matchCount == matchCountMax) {
				matchResult = RET_OK;
			}
		}
	}

	return matchResult;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  CtrlLogoff                                                              */
/*                                                                          */
/* 2.Description                                                            */
/*  ログオフコマンド受信時の処理	                                        */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	receivedCommand		受信バッファポインタ	                */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t CtrlLogoff(uint8_t* receivedCommand)
{
	int32_t ret = 0;

	if (!receivedCommand) {
		ret = -1;
		return ret;
	}

	_loginState = LOGOFF;

	if (Send(SEND_KEY_LOGOFF, NULL) != 0) {
		ret = -1;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibCF1CommCtrl_setStatus		                       	                */
/*                                                                          */
/* 2.Description                                                            */
/*  機器状態通知処理  						                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t LibCF1CommCtrl_setStatus(stCommStatus *data)
{
	int32_t ret = RET_OK;
	uint32_t i = 0;
	char key[ResMessageNumMax][10] = { // 機器状態のキー
			{ KEY_GAIN			},
			{ KEY_MUTE_MIC		},
			{ KEY_MUTE_SPK		},
			{ KEY_BLUETOOTH		},
			{ KEY_PRESET		},
			{ KEY_FOCUS_SET		},
			{ KEY_STANDBY		}
		};
	int32_t socket = GetSocket(); // ソケットを取得
	stCommStatus statusTmp;
	uint8_t* value;

	ConvertCommStatusApp2Lib(data, &statusTmp);

	if (socket != SOCKET_NULL) { // 接続されている場合のみ処理する

		// 自動ステイタスであれば、応答フラグをONにする
		ret |= SetAutoResponse(ResMessageNumMax, 10, key, &statusTmp);

		// ビームステアリングが音源推定(auto)であれば、指定時間超過で送信する
		ret |= SendFocusStatus();

		// 内部データを通知されたデータで上書きする
		_statusData = statusTmp;

		// 応答を送信する
		for(i = 0; i < ResMessageNumMax; i++)
		{
			// 制御応答フラグが立っているデータは必ず応答を送信する
			switch (i) {
			case ResMessageGain:		if (data->isGainResponse == 1)		_resMessage[i] = RES_ON;	break;
			case ResMessageMuteMic:		if (data->isMuteMicResponse == 1)	_resMessage[i] = RES_ON;	break;
			case ResMessageMuteSpk:		if (data->isMuteSpkResponse == 1)	_resMessage[i] = RES_ON;	break;
			case ResMessageBluetooth:	if (data->isBluetoothResponse == 1)	_resMessage[i] = RES_ON;	break;
			case ResMessageStandby:		if (data->isStandbyResponse == 1)	_resMessage[i] = RES_ON;	break;
			case ResMessagePreset:		if (data->isPresetResponse == 1)	_resMessage[i] = RES_ON;	break;
			case ResMessageFocusSet:	if (data->isFocusSetResponse == 1)	_resMessage[i] = RES_ON;	break;
			default:																					break;
			}

			if (_resMessage[i] == RES_ON) {
				if (i == ResMessageFocusSet) {
					value = MakeFocusSetData(&_statusData, key[i]);
				} else {
					value = GetCommStatus(&_statusData, key[i]);
				}

				if (Send(key[i], value) == -1) {
					ret = RET_NG;
				}

				_resMessage[i] = RES_OFF;		// 応答フラグクリア
			}
		}

		// タイムアウトしている場合はALIVEを送信する
		if (LibCTIMER_getTimeout(_aliveTimer) == 1) {
																	DBGPrintf("%s:TimeOutSendTimer Timeout\n", __FUNCTION__);
			if (Send(SEND_KEY_ALIVE, NULL) == -1) {
				ret = RET_NG;
			}
		}
	} else {
		// 内部データを通知されたデータで上書きする
		memcpy(&_statusData, &statusTmp, sizeof(stCommStatus));

		// 接続されていない場合はタイマーを更新する
		LibCTIMER_setTimer(_aliveTimer, ALIVE_TIMER_TIME);
	}
	memset(_resMessage, 0, sizeof(_resMessage));

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  SendCF1GetEvent		                             	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  制御イベント取得  						                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t SendCF1GetEvent(char* key, uint8_t* value)
{
	int32_t ret = RET_OK;
	uint32_t i = 0;
	stCommStatus status;
	struct {
		const char* key;
		int32_t*	isUpdated;
	} evList[] = {
		{ KEY_GAIN,		&(_controlDataFlg.gainUpdated)		},
		{ KEY_MUTE_MIC,	&(_controlDataFlg.muteMicUpdated)	},
		{ KEY_MUTE_SPK,	&(_controlDataFlg.muteSpkUpdated)	},
		{ KEY_BLUETOOTH,&(_controlDataFlg.bluetoothUpdated)	},
		{ KEY_PRESET,	&(_controlDataFlg.presetUpdated)	},
		{ KEY_STANDBY,	&(_controlDataFlg.standbyUpdated)	},
	};

	for (i = 0; i < (sizeof(evList) / sizeof(evList[0])); i++)
	{
		if (strcmp(key, evList[i].key) == 0) {
			ConvertCommStatusLib2App(&_controlData, &_controlDataFlg, &status);
			*value = *GetCommStatus(&status, evList[i].key);

			if (*(evList[i].isUpdated) != 0) {
				*(evList[i].isUpdated) = 0;	// 受信フラグOFF
			}

			break;
		}
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  SendCF1GetEventFocus                            	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  制御イベント取得  						                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t SendCF1GetEventFocus(uint8_t* mode, int8_t* direction, float* distance, uint8_t* unit)
{
	int32_t ret = RET_OK;
	stCommStatus status;
	struct stFocus {
		uint8_t		mode;			// ビームステアリング 0: 音源推定, 1: 音源固定
		int8_t		direction;		// ビームステアリング 音源固定の角度
		uint8_t		unit;			// ビームステアリング 単位系 0: inch, 1: cm
		uint8_t		dummay;			// 4byteアライメント用のダミー
		float		distance;		// ビームステアリング 音源固定の距離(m)
	} *focus;

	ConvertCommStatusLib2App(&_controlData, &_controlDataFlg, &status);
	focus = (struct stFocus *)GetCommStatus(&status, KEY_FOCUS_SET);

	*mode = focus->mode;
	*direction = focus->direction;
	*distance = focus->distance;
	*unit = focus->unit;

	if (_controlDataFlg.focusSetUpdated != 0) {
		_controlDataFlg.focusSetUpdated = 0;	// 受信フラグOFF
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  LibCF1CommCtrl_DefBTInvalid                        	                    */
/*  LibCF1CommCtrl_DefBtOn                                                  */
/*  LibCF1CommCtrl_DefBtOff                                                 */
/*  LibCF1CommCtrl_DefBtModeOff                                             */
/*  LibCF1CommCtrl_DefBtModePairing                                         */
/*  LibCF1CommCtrl_DefBtModeConnected                                       */
/*                                                                          */
/* 2.Description                                                            */
/*  各種の定義値を返す　　　　　　                                          */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  LibCF1CommCtrl_DefBTInvalid        Bluetooth制御:指示なし               */
/*  LibCF1CommCtrl_DefBtOn             Bluetooth制御:ON                     */
/*  LibCF1CommCtrl_DefBtOff            Bluetooth制御:OFF                    */
/*  LibCF1CommCtrl_DefBtModeOff        Bluetoothモード:OFF                  */
/*  LibCF1CommCtrl_DefBtModePairing    Bluetoothモード:ペアリング登録中     */
/*  LibCF1CommCtrl_DefBtModeConnected  Bluetoothモード:接続中               */
/*                                                                          */
/*--------------------------------------------------------------------------*/
uint32_t LibCF1CommCtrl_DefBtInvalid( void )
{
	return BLUETOOTH_INVALID;
}

uint32_t LibCF1CommCtrl_DefBtOn( void )
{
	return BLUETOOTH_ON;
}

uint32_t LibCF1CommCtrl_DefBtOff( void )
{
	return BLUETOOTH_OFF;
}

uint32_t LibCF1CommCtrl_DefBtStatusOff( void )
{
	return BT_STATUS_OFF;
}

uint32_t LibCF1CommCtrl_DefBtStatusPairing( void )
{
	return BT_STATUS_PAIRING;
}

uint32_t LibCF1CommCtrl_DefBtStatusConnected( void )
{
	return BT_STATUS_CONNECTED;
}


/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  SetGain                 			            	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  チャンネルフェーダーゲイン保存処理		                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*	uint8_t	value	0x00 - 0x3F	:	ポジション指定							*/
/*					0x41 - 0x5F	:	ステップ指定(UP)						*/
/*					0x61 - 0x7F	:	ステップ指定(DOWN)						*/
/*                                                                          */
/* 4.Return                                                                 */
/*  int8_t	0: OK, -1: NG(不正値)											*/
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int8_t SetGain(uint8_t value)
{
	int8_t	ret = 0;
	uint8_t	gain = 0;

	// WEB MUTE ONの場合は無効
	if (_statusData.muteSpk == 1) {
		ERRPrintf("%s:step up/down(value: 0x%02x) is disable on web mute.\n",
			__FUNCTION__, value);
		ret = -1;
	}

	if (value <= 0x3F) {
		// ポジション指定

		_controlData.gain = value;
	}
	else {
		// ステップ指定

		if ((value >= 0x41) && (value <= 0x5F)) {
			// UP(0x41: 1pos up, 0x5F: 31pos up)

			gain = _statusData.gain + (value - 0x40);
			_controlData.gain = ((gain > 0x3F) ? 0x3F : gain);
		}
		else if ((value >= 0x61) && (value <= 0x7F)) {
			// DOWN(0x61: 1pos down, 0x7F: 31pos down)

			if (_statusData.gain > (value - 0x60)) {
				gain = _statusData.gain - (value - 0x60);
			}
			else {
				gain = 0;
			}

			_controlData.gain = gain;
		}
		else {
			ERRPrintf("%s:illegal value:0x%02x\n", __FUNCTION__, value);
			ret = -1;
		}
	}
																	DBGPrintf("_controlData.gain:%d\n", _controlData.gain);
	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name																	*/
/*  SetMuteMic																*/
/*																			*/
/* 2.Description															*/
/*  MICチャンネルミュートON/OFF保存処理										*/
/*																			*/
/* 3.Parameter(s)															*/
/*  uint8_t	value	0x00: チャンネルミュート OFF							*/
/*					0x01: チャンネルミュート ON								*/
/*																			*/
/* 4.Return																	*/
/*  int8_t	0: OK, -1: NG(不正値)											*/
/*																			*/
/*--------------------------------------------------------------------------*/
static int8_t SetMuteMic(uint8_t value)
{
	int8_t ret = 0;

	if (value <= 0x01) {
		_controlData.muteMic = value;
	} else {
		ERRPrintf("%s:illegal value:0x%02x\n", __FUNCTION__, value);
		ret = -1;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name																	*/
/*  SetMuteSpk																*/
/*																			*/
/* 2.Description															*/
/*  SPEAKERチャンネルミュートON/OFF保存処理									*/
/*																			*/
/* 3.Parameter(s)															*/
/*  uint8_t	value	0x00: チャンネルミュート OFF							*/
/*					0x01: チャンネルミュート ON								*/
/*																			*/
/* 4.Return																	*/
/*  int8_t	0: OK, -1: NG(不正値)											*/
/*																			*/
/*--------------------------------------------------------------------------*/
static int8_t SetMuteSpk(uint8_t value)
{
	int8_t ret = 0;

	if (value <= 0x01) {
		_controlData.muteSpk = value;
	}
	else {
		ERRPrintf("%s:illegal value:0x%02x\n", __FUNCTION__, value);
		ret = -1;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name																	*/
/*  SetBluetooth														*/
/*																			*/
/* 2.Description															*/
/*  BluetoothModeのON/OFF保存処理											*/
/*																			*/
/* 3.Parameter(s)															*/
/*  uint8_t	value	OFF	(Bluetooth接続を切断またはペアリング登録を中断)		*/
/*					ON	(ペアリング登録の開始)								*/
/*																			*/
/* 4.Return																	*/
/*  int8_t	0: OK, -1: NG(不正値)											*/
/*																			*/
/*--------------------------------------------------------------------------*/
static int8_t SetBluetooth(uint8_t value)
{
	int8_t ret = RET_OK;

	if (value < BLUETOOTH_PARAM_MAX) {
		_controlData.bluetoothMode = value;
	}
	else {
		ERRPrintf("%s:illegal value:0x%02x\n", __FUNCTION__, value);
		ret = RET_NG;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  SetPreset                 			            	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  プリセットメモリ呼出し保存処理		  	         	                    */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t	value	0x00: プリセット1                                       */
/*					0x01: プリセット2                                       */
/*                                                                          */
/* 4.Return                                                                 */
/*  int8_t	0: OK, 1: NG(不正値)											*/
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int8_t SetPreset(uint8_t value)
{
	int8_t ret = 0;

	if (value <= 0x01) {
		_controlData.preset = value;
	} else {
		ERRPrintf("%s:illegal value:0x%02x\n", __FUNCTION__, value);
		ret = -1;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  SetStandby                 			            	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  スタンバイモード切替保存処理		  	                              	*/
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t	value	0x00: スタンバイモード OFF                              */
/*					0x01: スタンバイモード ON                               */
/*                                                                          */
/* 4.Return                                                                 */
/*  int8_t	0: OK, -1: NG(不正値)											*/
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int8_t SetStandby(uint8_t value)
{
	int8_t ret = 0;

	if (value <= 0x01) {
		_controlData.standby = value;
	} else {
		ERRPrintf("%s:illegal value:0x%02x\n", __FUNCTION__, value);
		ret = -1;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  SetFocusSetting            			            	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  ビームステアリング設定保存処理		  	                              	*/
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t		mode	0x00: 音源推定モード	                            */
/*						0x01: 音源固定モード                                */
/*  int8_t		dir		音源推定モード: なし	          		            */
/*						音源固定モード: 角度           		                */
/*  uint16_t	dis		音源推定モード: なし        		                */
/*						音源固定モード: 距離		                        */
/*  uint16_t	uint	0x00: inch				                            */
/*						0x01: cm            			                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  int8_t	0: OK, -1: NG(不正値)											*/
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int8_t SetFocusSetting(uint8_t mode, int8_t dir, uint16_t dis, uint8_t unit)
{
	int8_t ret = RET_OK;
	float distance;

	// auto
	if (mode == FOCUS_AUTO) {
		// 制御指示データに設定する
		_controlData.focus_mode = mode;
	}
	// manual
	else if (mode == FOCUS_MANUAL) {
		// 角度(範囲チェック)
 		if ((dir < FOCUS_DIR_MIN) || (dir > FOCUS_DIR_MAX)) {
			ERRPrintf("%s:illegal dir:0x%02x\n", __FUNCTION__, dir);
			ret = RET_NG;
		}

		// 単位(範囲チェック)
		if (unit > FOCUS_UNIT_CM) {
			ERRPrintf("%s:illegal unit:0x%02x\n", __FUNCTION__, unit);
			ret = RET_NG;
		}

		// 0.1cm or 0.1inch を m に変換する
		if (unit == FOCUS_UNIT_INCH) {
			distance = ConvertInchToM(dis);
		} else {
			distance = ConvertCmToM(dis);
		}
		// 距離(範囲チェック)
 		if ((distance < FOCUS_DIS_MIN) || (distance > FOCUS_DIS_MAX)) {
			ERRPrintf("%s:illegal dis:0x%04x\n", __FUNCTION__, dis);
			ret = RET_NG;
		}

		if (ret == RET_OK) {
			// 制御指示データに設定する
			_controlData.focus_mode = mode;
			_controlData.focus_direction = dir;
			_controlData.focus_distance = distance;
			_controlData.focus_unit = unit;
		} else {
			// 制御指示データに設定しない
		}
	}
	else {
		// 制御指示データには設定しない
		ERRPrintf("%s:illegal mode:0x%02x\n", __FUNCTION__, mode);
		ret = RET_NG;
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  SetAutoResponse		                             	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  自動ステイタスによる応答フラグ変更                                      */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint32_t		key_size1	キー情報の配列サイズ1					    */
/*  uint32_t		key_size2	キー情報の配列サイズ2					    */
/*  char			key[][]		自動ステイタス通知の対象となるキー情報      */
/*  stCommStatus	statusTmp	更新前のステイタス情報		                */
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t	0: OK, -1: NG(不正値)											*/
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t SetAutoResponse(uint32_t key_size1, uint32_t key_size2, char key[key_size1][key_size2], stCommStatus* statusTmp)
{
	int32_t ret = RET_OK;
	uint32_t i = 0;
	const uint8_t *beforeData;
	const uint8_t *renewData;

	// 自動応答ONかつログイン中の場合、値が変更されているデータを応答ONにする
	if ((_autoResponse == AUTO_STATUS_ON)
	 && (_loginState == LOGIN)) {
		for(i = 0; i < ResMessageNumMax; i++)
		{
			// プリセットは変化に関係なく応答する(仕様)
			if (i == ResMessagePreset) {
				if (statusTmp->isPresetAutoResponse) {
																	DBGPrintf("%s:RES_ON[%s] before = %u, after = %u\n", __FUNCTION__, key[i], beforeData, renewData);
					_resMessage[i] = RES_ON;
				}
			}
			// ビームステアリング以外を対象とする
			else if (i != ResMessageFocusSet) {
				beforeData = GetCommStatus(&_statusData, key[i]);
				renewData = GetCommStatus(statusTmp, key[i]);

				if (*beforeData != *renewData) {
																	DBGPrintf("%s:RES_ON[%s] before = %u, after = %u\n", __FUNCTION__, key[i], beforeData, renewData);
					_resMessage[i] = RES_ON;
				}
			}
		}
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  SendFocusStatus		                             	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  ビームステアリング通知送信	                                            */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t	0: OK, -1: NG(不正値)											*/
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t SendFocusStatus(void)
{
	int32_t ret = RET_OK;

	// ビームステアリング応答ONかつログイン中、即時 or 指定間隔を超過していればデータを応答ONにする
	if ((_focusStatus.autoResponse == AUTO_STATUS_ON)
	 && (_loginState == LOGIN)) {

		// 即時応答は、X,Y座標を送信する
		if (_focusStatus.interval == FOCUS_NO_TIME) {
			if (Send(SEND_KEY_FOCUS, MakeFocusData(&_statusData, SEND_KEY_FOCUS, _focusStatus.autoResponseUnit)) != 0) {
				ret |= RET_NG;
			}
		}
		// 時間指定は、タイムアウトした時にX,Y座標を送信する
		else {
			if (LibCTIMER_getTimeout(_focusTimer) == 1) {
				if (Send(SEND_KEY_FOCUS, MakeFocusData(&_statusData, SEND_KEY_FOCUS, _focusStatus.autoResponseUnit)) != 0) {
					ret |= RET_NG;
				}

				LibCTIMER_setTimer(_focusTimer, _focusStatus.interval * FOCUS_TIME_UNIT);
			}
		}
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  UpdateFocusTimer		                          	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  ビームステアリングのタイマーを更新する                                  */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static void UpdateFocusTimer(void)
{
	// Autoの場合はタイマーを設定する
	if ((_focusStatus.autoResponse == AUTO_STATUS_ON) && (_focusStatus.interval != FOCUS_NO_TIME)) {
		LibCTIMER_setTimer(_focusTimer, _focusStatus.interval * FOCUS_TIME_UNIT);
	}
	// manualの場合はタイマーを解除する
	else {
		LibCTIMER_cancelTimer(_focusTimer);
	}
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  MakeFocusData				                       	         	        */
/*                                                                          */
/* 2.Description                                                            */
/*  ビームステアリング通知データ作成(ControllerIFで使用する)                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t	0: OK, -1: NG(不正値)											*/
/*                                                                          */
/*--------------------------------------------------------------------------*/
static uint8_t* MakeFocusData(stCommStatus* target, const char* key, uint8_t unit)
{
	// この構造体はstCommStatusを参考に作成すること
	struct stFocus {
		float		x;
		float		y;
	} *focus;

	static struct stFocusData {
		int16_t		x;
		uint16_t	y;
		uint8_t		unit;
		uint8_t		dummay;
	} focusData;

	focus = (struct stFocus *)GetCommStatus(target, key);

	focusData.unit = unit;
	if (focusData.unit == FOCUS_UNIT_INCH) {
		focusData.x =  (int16_t)( ConvertMToInch(focus->x) );
		focusData.y = (uint16_t)( ConvertMToInch(focus->y) );
	}
	else {
		focusData.x =  (int16_t)( ConvertMToCm(focus->x) );
		focusData.y = (uint16_t)( ConvertMToCm(focus->y) );
	}

	return (uint8_t*)(&focusData);
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  MakeFocusSetData                	             	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  ビームステアリング設定データ作成(ControllerIFで使用する)                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t	0: OK, -1: NG(不正値)											*/
/*                                                                          */
/*--------------------------------------------------------------------------*/
static uint8_t* MakeFocusSetData(stCommStatus* target, const char* key)
{
	// この構造体はstCommStatusを参考に作成すること
	struct stFocusSet {
		uint8_t		mode;
		int8_t		dir;
		uint8_t		unit;
		uint8_t		dummy;
		float		dis;
	} *focusSet;

	static struct stFocusSetData {
		uint8_t		mode;
		int8_t		dir;
		uint8_t		unit;
		uint8_t		dummy;
		uint16_t	dis;
	} focusSetData;

	focusSet = (struct stFocusSet *)GetCommStatus(target, key);

	focusSetData.mode = focusSet->mode;
	focusSetData.dir = focusSet->dir;
	focusSetData.unit = focusSet->unit;
	if (focusSetData.unit == FOCUS_UNIT_INCH) {
		focusSetData.dis = (uint16_t)( ConvertMToInch(focusSet->dis) );
	}
	else {
		focusSetData.dis = (uint16_t)( ConvertMToCm(focusSet->dis) );
	}

	return (uint8_t*)(&focusSetData);
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  GetCommStatus		                             	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  機器状態取得	                                                        */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static uint8_t* GetCommStatus(stCommStatus* target, const char* key)
{
	uint8_t* value = NULL;

	if (strcmp(key, KEY_GAIN) == 0) {					// チャンネルフェーダーゲイン
		value = &(target->gain);
	} else if (strcmp(key, KEY_MUTE_MIC) == 0) {		// MicチャンネルミュートON/OFF
		value = &(target->muteMic);
	} else if (strcmp(key, KEY_MUTE_SPK) == 0) {		// SpeakerチャンネルミュートON/OFF
		value = &(target->muteSpk);
	} else if (strcmp(key, KEY_BLUETOOTH) == 0) {		// BluetoothモードON/OFF
		value = &(target->bluetoothMode);
	} else if (strcmp(key, KEY_PRESET) == 0) {			// プリセット
		value = &(target->preset);
	} else if (strcmp(key, KEY_STANDBY) == 0) {			// スタンバイモード切替
		value = &(target->standby);
	} else if (strcmp(key, SEND_KEY_FOCUS) == 0) {		// ビームステアリング
		value = (uint8_t*)&(target->focus_x);			// 　X, Y座標の先頭アドレスを格納する
	} else if (strcmp(key, KEY_FOCUS_SET) == 0) {		// ビームステアリング設定
		value = (uint8_t*)&(target->focus_mode);		// 　設定値の先頭アドレスを格納する
	} else {
																	DBGPrintf("%s:key(%10s) not found.\n", __FUNCTION__, key);
	}
	return value;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  Send				                             	                    */
/*                                                                          */
/* 2.Description                                                            */
/*  送信処理		  						                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t Send(const char* command, const uint8_t* value)
{
	int32_t ret = 0;
	int32_t socket = GetSocket();

	if (socket != SOCKET_NULL) // 接続されている場合は送信する
	{
		ret |= LibCTIMER_setTimer(_aliveTimer, ALIVE_TIMER_TIME);
		ret |= SendControlCommand(socket, command, value);
	}

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  DisconnectedCallback                                                    */
/*                                                                          */
/* 2.Description                                                            */
/*  通信切断時に呼び出されるコールバック処理	                            */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static void DisconnectedCallback(void)
{
	_loginState = LOGOFF;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  ConvertEndian        		                                            */
/*                                                                          */
/* 2.Description                                                            */
/*  エンディアンの変換							                            */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t*	src			変換元ポインタ									*/
/*  uint8_t*	dst			変換先ポインタ									*/
/*  uint32_t	size		変換サイズ										*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t ConvertEndian(uint8_t* src, uint8_t* dst, uint32_t size)
{
	uint32_t count;

	// 範囲チェック
	if (size < 2) {
		return RET_NG;
	}

	for(count = 0; count < size; count++) {
		dst[count] = src[(size - 1) - count];
	}

	return RET_OK;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  ConvertInchToM					                       	                */
/*                                                                          */
/* 2.Description                                                            */
/*  単位変換(0.1inch -> m)					                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*	uint16_t		x		変換元の値										*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static float ConvertInchToM(uint16_t x)
{
	return (x * 2.5 / 10.0 / 100.0);
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  ConvertCmToInch					                       	                */
/*                                                                          */
/* 2.Description                                                            */
/*  単位変換(0.1cm -> m)					                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*	uint16_t		x		変換元の値										*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static float ConvertCmToM(uint16_t x)
{
	return (x / 10.0 / 100.0);
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  round_f							                       	                */
/*                                                                          */
/* 2.Description                                                            */
/*  四捨五入								                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*	float			x		四捨五入する値									*/
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t	計算結果														*/
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t round_f(float x)
{
    if ( x >= 0.0 ) {
        return (int32_t)((x * 10 + 5)/10);
    } else {
        return (int32_t)(-1.0 * ((fabs(x) * 10 + 5)/10));
    }
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  ConvertMToCm					                       	                */
/*                                                                          */
/* 2.Description                                                            */
/*  単位変換(m -> 0.1cm)					                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*	float			x		変換元の値										*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t ConvertMToCm(float x)
{
	return round_f(x * 100.0 * 10.0);
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  ConvertMToInch					                       	                */
/*                                                                          */
/* 2.Description                                                            */
/*  単位変換(m -> 0.1inch)					                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*	float			x		変換元の値										*/
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t ConvertMToInch(float x)
{
	return round_f(x * 100.0 * 10.0 / 2.5);
}









// 以下削除予定(動作確認用)
void main(void)
{
	stCommStatus status;
	memset(&status, 0x00, sizeof(status));

	LibCF1CommCtrl_Init("/home/toa/TOA_AM-CF1/Application/data/config/config_userpass.txt", 193, 1);

	status = _statusData;
	status.focus_x = -3.1234;
	status.focus_y = 6.8765;
//	status.focus_x = 3.8765;
//	status.focus_y = -0.1234;
//	status.focus_x = 0.1234;
//	status.focus_y = 3.8765;

#if 0 // 認証チェック用関数のテスト
	int32_t matchNameResult = RET_NG;
	matchNameResult = CheckloginStr("admint", USERNAME_MAX, PADDING_CHAR, "admin");
	printf("test1  = %d\n", matchNameResult);
	matchNameResult = CheckloginStr("admin", USERNAME_MAX, PADDING_CHAR, "admin");
	printf("test2  = %d\n", matchNameResult);
	matchNameResult = CheckloginStr("admim", USERNAME_MAX, PADDING_CHAR, "admin");
	printf("test3  = %d\n", matchNameResult);
	matchNameResult = CheckloginStr("amcf1guest", USERNAME_MAX, PADDING_CHAR, "admin");
	printf("test4  = %d\n", matchNameResult);
	matchNameResult = CheckloginStr("1234567890123456", USERNAME_MAX, PADDING_CHAR, "admin");
	printf("test5  = %d\n", matchNameResult);
	matchNameResult = CheckloginStr("", USERNAME_MAX, PADDING_CHAR, "admin");
	printf("test6  = %d\n", matchNameResult);
	matchNameResult = CheckloginStr("amcf1", USERNAME_MAX, PADDING_CHAR, "amcf1");
	printf("test7  = %d\n", matchNameResult);
	matchNameResult = CheckloginStr("amcf1guest", USERNAME_MAX, PADDING_CHAR, "amcf1guest");
	printf("test8  = %d\n", matchNameResult);
	matchNameResult = CheckloginStr("", USERNAME_MAX, PADDING_CHAR, "");
	printf("test9  = %d\n", matchNameResult);
	matchNameResult = CheckloginStr("admin", USERNAME_MAX, PADDING_CHAR, "");
	printf("test10 = %d\n", matchNameResult);
	matchNameResult = CheckloginStr("amcf1guest", USERNAME_MAX, PADDING_CHAR, "");
	printf("test11 = %d\n", matchNameResult);

	LibCF1CommCtrl_Deinit();
																	DBGPrintf("End\n");
	return;
#endif

#if 0 // 変換テスト
	int count;
	int cm = 0, inch = 0;
	float m = -0.0010;

//	for(count = 0; count <= 6000; count++) {
	for(count = 0; count <= 200; count++) {
		printf("--------------------------\n");
		printf("cm = %d, inch = %d, m = %f\n", cm, inch, m);

		printf("inch -> m : %f\n", ConvertInchToM(inch));
		printf("cm   -> m : %f\n", ConvertCmToM(cm));
		printf("m ->   cm : %d\n", ConvertMToCm(m));
		printf("m -> inch : %d\n", ConvertMToInch(m));

		cm++;
		inch++;
//		m += 0.001;
		m += 0.0001;
	}
#endif

	while(1) {
		LibCF1CommCtrl_Control();
		if(_controlDataFlg.focusSetUpdated == 1) {
			status.isFocusSetResponse = 1;
		}
		else {
			status.isFocusSetResponse = 0;
		}

		usleep(100 * 1000);	// 100msec

		if(_controlDataFlg.gainUpdated == 1)		SendCF1GetEvent(KEY_GAIN, &status.gain);
		if(_controlDataFlg.muteMicUpdated == 1)		SendCF1GetEvent(KEY_MUTE_MIC, &status.muteMic);
		if(_controlDataFlg.muteSpkUpdated == 1)		SendCF1GetEvent(KEY_MUTE_SPK, &status.muteSpk);
		if(_controlDataFlg.bluetoothUpdated == 1)	SendCF1GetEvent(KEY_BLUETOOTH, &status.bluetoothMode);
		if(_controlDataFlg.presetUpdated == 1)		SendCF1GetEvent(KEY_STANDBY, &status.standby);
		if(_controlDataFlg.standbyUpdated == 1)		SendCF1GetEvent(KEY_PRESET, &status.preset);
		if(_controlDataFlg.focusSetUpdated == 1)	SendCF1GetEventFocus(&status.focus_mode, &status.focus_direction, &status.focus_distance, &status.focus_unit);
		status.preset = 0x10;

		if(status.isFocusSetResponse == 1) {
			status.focus_mode--;
		}

		LibCF1CommCtrl_setStatus(&status);

#if 0 // TCP-IPの切断確認
		if(_statusData.standby == 1) {
																	DBGPrintf("debug end\n");
			LibCF1CommCtrl_Deinit();

			sleep(10);

			_statusData.standby = 0;
			LibCF1CommCtrl_Init("/home/toa/TOA_AM-CF1/Application/data/config/config_userpass.txt", 193);
		}
#endif

	}
	LibCF1CommCtrl_Deinit();
																	DBGPrintf("End\n");
}
