/*--------------------------------------------------------------------------*/
/*																			*/
/*	TCPIPReceiver.c (Rev 1.00)												*/
/*																			*/
/*	1.概要																	*/
/*		AM-CF1外部制御プロトコル											*/
/*																			*/
/*	2.改版履歴																*/
/*																			*/
/*																			*/
/*--------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <pthread.h>
#include <signal.h>
#include <sys/sem.h>
#include <errno.h>
#include "ControllerIF.h"
#include "TCPIPReceiver.h"
#include "UtilityforDebug.h"

#define PORT_NUM				3000	// ポート番号
#define SELECT_TIMEOUT_SEC		60		// セレクト待ちタイムアウト(秒)

static void		*TCPIPReceiverThread(void* p);
static int32_t 	TCPIPReceivedCommand(void);
static int32_t	SelectReceivePacket(uint8_t* pReceiveData, int32_t length, uint8_t* pCommand);
static void		RegistReceiveCommand(uint8_t* data, uint32_t length);
static int32_t	AccessSemaphore(int32_t isGet);
static void		SetSocketConnected(int32_t targetSocket);

// セマフォ用の構造
union semun {
    int32_t			val;
    struct semid_ds *buf;
    uint16_t 		*array;
    struct seminfo 	*_buf;
};

static pthread_t		_pthread; 		// TCPIPReceiverスレッド
static int32_t			_srcSocket;		// ソケット(自分)
static int32_t			_sam;		    // セマフォ
static stReceiveBuffer	_receiveBuffer; // 受信バッファ

// TCPIP接続用の変数
static fd_set			_fds;			// fd_set
static int32_t			_targetSocket;	// 送受信先のソケット

// コールバック用の変数
static DISCONNECTED_CALLBACK	pDisconnectedFunc;	// 通信切断時のコールバック関数ポインタ

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  TCPIPReceiver_Init	                                                    */
/*                                                                          */
/* 2.Description                                                            */
/*  スレッド初期化処理						                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  DISCONNECTED_CALLBACK	pfunc	登録するコールバック関数	            */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t TCPIPReceiver_Init(DISCONNECTED_CALLBACK pfunc)
{
	union semun semcmd; // semctl へ渡すコマンド用の共用体
	
	_srcSocket = SOCKET_NULL;
	_targetSocket = SOCKET_NULL;

	// 受信バッファ
	memset(&_receiveBuffer, 0, sizeof(_receiveBuffer));
	
	// セマフォを取得する
    _sam = semget(IPC_PRIVATE, 1, 0666|IPC_CREAT);
	if (_sam == -1) {
		ERRPrintf("[%s] failed to semget(). errno: %d\n", __FUNCTION__, errno);
		errno = 0;
		return -1;
	}

    // セマフォに初期値を設定する(この値が、同時実行を許可する数になる)
    semcmd.val = 1;
	if (semctl(_sam, 0, SETVAL, semcmd) == -1) {
		ERRPrintf("[%s] failed to semctl(). errno: %d\n", __FUNCTION__, errno);
		errno = 0;
		return -1;
	}

	if (pthread_create(&_pthread, NULL, &TCPIPReceiverThread, NULL) != 0) {
		ERRPrintf("[%s] failed to pthread_create(). errno: %d\n", __FUNCTION__, errno);
		errno = 0;
		return -1;
	}
	
	// 通信切断時のコールバック関数を登録する
	pDisconnectedFunc = pfunc;
	
	return 0;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  AccessSemaphore					                                        */
/*                                                                          */
/* 2.Description                                                            */
/* セマフォ制御																*/
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t AccessSemaphore(int32_t isGet)
{
	struct sembuf semb; // semop へ渡す設定内容の構造体

	semb.sem_num = 0;
	semb.sem_op = ((isGet == 1) ? -1 : 1); // セマフォから権限を取得/返還する
	semb.sem_flg = SEM_UNDO;

	if (semop(_sam, &semb, 1) == -1) {
		return -1;
	}

	return 0;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  GetSocket						                                        */
/*                                                                          */
/* 2.Description                                                            */
/*  ソケットを取得する						                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t GetSocket()
{
	int32_t targetSocket = SOCKET_NULL;

	if (AccessSemaphore(1) == 0) { // セマフォ取得
		targetSocket = _targetSocket;
		AccessSemaphore(0); // セマフォ返還
	}

	return targetSocket;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  TCPIPReceiver_Deinit				                                    */
/*                                                                          */
/* 2.Description                                                            */
/*  スレッド終了処理						                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t TCPIPReceiver_Deinit(void)
{
	int32_t ret = 0;
	int32_t socket = GetSocket();
	
	// スレッドの生存確認
	if(pthread_kill(_pthread, 0) != ESRCH)
	{
		ret = pthread_cancel(_pthread); 			// TCPListenerスレッドの削除
																			DBGPrintf("[%s] pthread cancel : return = %d\n", __FUNCTION__, ret);
		ret = pthread_join(_pthread, NULL);			// TCPListenerスレッドの終了待ち
																			DBGPrintf("[%s] pthread kill : return = %d\n", __FUNCTION__, ret);
	}

	// ソケットの終了
	if (socket != SOCKET_NULL) {
		if (close(socket) == -1) {					// 接続済みソケットのClose
			printf("[%s] failed to close(client socket). errno: %d\n",
				__FUNCTION__, errno);
			errno = 0;
			ret = -1;
		}
		SetSocketConnected(SOCKET_NULL);
	}
	if (_srcSocket != SOCKET_NULL) {
		if (close(_srcSocket) == -1) {				// Listen中ソケットのClose
			printf("[%s] failed to close(listener socket). errno: %d\n",
				__FUNCTION__, errno);
			errno = 0;
			ret = -1;
		}
		_srcSocket = SOCKET_NULL;
	}

	pDisconnectedFunc();
	pDisconnectedFunc = NULL;

	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  GetReceiveCommand				                                        */
/*                                                                          */
/* 2.Description                                                            */
/*  受信したコマンドを引き渡す				                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
void GetReceiveCommand(stReceiveBuffer *buf)
{
	if (AccessSemaphore(1) == 0) { // セマフォ取得
		memcpy(buf, &_receiveBuffer, sizeof(_receiveBuffer));
		memset(&_receiveBuffer, 0, sizeof(_receiveBuffer));
		AccessSemaphore(0); // セマフォ返還
	}
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  TCPIPReceiverThread				                                        */
/*                                                                          */
/* 2.Description                                                            */
/*  TCPIPReceiverスレッド処理				                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static void *TCPIPReceiverThread(void* p)
{
	int32_t  status = 0;
	int32_t	 targetSocket = SOCKET_NULL;
	char command[] = SEND_KEY_STARTUP;

	/*------------------------------------------------------------------*/
	/*　接続通知を送信する（コントローラー側で処理するかも）			*/
	/*------------------------------------------------------------------*/

	while(1){

		targetSocket = TCPIPReceiverSocketSetup();
		if(targetSocket != SOCKET_NULL){
			
			SetSocketConnected(targetSocket);
			SendControlCommand(targetSocket, command, NULL);
			
			while(targetSocket != SOCKET_NULL){
				status = TCPIPReceivedCommand();					/* パケット抽出	*/
				if(status == 1){
					/* 処理なし */
				}
				else if(status == 0){
					SetSocketConnected(SOCKET_NULL);
					break;
				}
				else{
					/* システムコールに失敗した場合はスレッドを終了する*/
		  			SetSocketConnected(SOCKET_NULL);
					return NULL;
				}
			}
		}
		/*--------------------------------------------------------------*/
		/*　初期化失敗した場合はスレッドを終了する						*/
		/*--------------------------------------------------------------*/
		else{
			break;
		};
	}

	return NULL;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  TCPIPReceiverSocketSetup				                                */
/*                                                                          */
/* 2.Description                                                            */
/*  TCPIPReceiverスレッドソケットの初期化処理                               */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t target                    	                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
int32_t TCPIPReceiverSocketSetup(void)
{

	// ポート番号
	uint16_t 				port = PORT_NUM;
	struct 					sockaddr_in srcAddr;
	struct 					sockaddr_in dstAddr;
	int32_t 				dstAddrSize = sizeof(dstAddr);
	int32_t 				ret = 0;
	int32_t		   			target = SOCKET_NULL;
	int32_t					yes = 1;
	
	if (_srcSocket == SOCKET_NULL) {
		/*----------------------------------------------------------------------*/
		/* ソケットを生成する													*/
		/*----------------------------------------------------------------------*/
		memset(&srcAddr, 0, sizeof(srcAddr));
		srcAddr.sin_port = htons(port);
		srcAddr.sin_family = AF_INET;
		srcAddr.sin_addr.s_addr = htonl(INADDR_ANY);
		_srcSocket = socket(AF_INET, SOCK_STREAM, 0);

		if (_srcSocket < 0)
		{
			/* 異常処理															*/
																	DBGPrintf("socket failed:%d\n", _srcSocket);
			return SOCKET_NULL;
		}

		/*----------------------------------------------------------------------*/
		/* アドレスの再使用設定													*/
		/*----------------------------------------------------------------------*/
		ret = setsockopt(_srcSocket, SOL_SOCKET, SO_REUSEADDR, (const uint8_t *)&yes, sizeof(yes));
		if (ret < 0)
		{
			/* 異常処理															*/
																	DBGPrintf("setsockopt(REUSEADDR) failed:%d\n", ret);
			close(_srcSocket);
			_srcSocket = SOCKET_NULL;
			return SOCKET_NULL;
		}

		/*----------------------------------------------------------------------*/
		/* ソケットをバインドする												*/
		/*----------------------------------------------------------------------*/
		ret = bind(_srcSocket, (struct sockaddr *) &srcAddr, sizeof(srcAddr));
		if (ret < 0)
		{
			/* 異常処理															*/
																	DBGPrintf("bind failed:%d\n", ret);
			close(_srcSocket);
			_srcSocket = SOCKET_NULL;
			return SOCKET_NULL;
		}

		/*----------------------------------------------------------------------*/
		/* 接続を許可する														*/
		/*----------------------------------------------------------------------*/
		ret = listen(_srcSocket, 1);
		if (ret < 0)
		{
																	DBGPrintf("listen failed:%d\n", ret);
			close(_srcSocket);
			_srcSocket = SOCKET_NULL;
			return SOCKET_NULL;
		}
	}

	/*----------------------------------------------------------------------*/
	/* 接続を受け付ける														*/
	/*----------------------------------------------------------------------*/
																	DBGPrintf("Wait Connect\n");
	target = accept(_srcSocket, (struct sockaddr *) &dstAddr, &dstAddrSize);
	if(target  < 0)
	{
																	DBGPrintf("accept failed:%d\n", ret);
		close(_srcSocket);
		_srcSocket = SOCKET_NULL;
		return SOCKET_NULL;
	}

	FD_ZERO(&_fds);
	FD_SET(target, &_fds);
	
																	DBGPrintf("Connected from %s\n", inet_ntoa(dstAddr.sin_addr));

	// 不要なListenソケットをクローズする
	if (_srcSocket != SOCKET_NULL) {
		if (close(_srcSocket) == -1) {				// Listen中ソケットのClose
			printf("[%s] failed to close(listener socket). errno: %d\n",
				__FUNCTION__, errno);
		}
		_srcSocket = SOCKET_NULL;
	}

	return target;
}


/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  TCPIPReceivedCommand			                                        */
/*                                                                          */
/* 2.Description                                                            */
/*  TCPIPReceiverスレッド処理				                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t       0:受信失敗  1:受信                                            */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t TCPIPReceivedCommand(void)
{
	int32_t ret = 0;
	int32_t status = 0;
	int32_t length = 0;
	uint8_t buffer[LEN_COMMAND];
	uint8_t command[LEN_COMMAND];
 	struct timeval tv;
	int32_t position = 0;
	int32_t socket = GetSocket();
	fd_set fds;
	memset(command, 0, sizeof(command));
	memset(buffer, 0, sizeof(buffer));
																	DBGPrintf("[%s] TCPIPReceivedCommand:%d\n", __FUNCTION__, socket);

	// 10分タイムアウトで受信待ち
	tv.tv_sec 	= SELECT_TIMEOUT_SEC;
	tv.tv_usec 	= 0;

	memcpy(&fds, &_fds, sizeof(fd_set));

	ret = select(socket + 1, &fds, NULL, NULL, &tv);
	if (ret != 0) {
		if (FD_ISSET(socket, &fds)) {
			length = recv(socket, buffer, LEN_COMMAND, 0); 
	    	if(length > 0){
																	DBGPrintf("[%s] ReceivedPacket:%d\n", __FUNCTION__, length);
				
				/*----------------------------------------------------------*/
				/* 受信データからパケットを抽出する							*/
				/*----------------------------------------------------------*/
				status = SelectReceivePacket(&buffer[0], length, &command[0]);
				
				/*----------------------------------------------------------*/
				/* パケットを共有データに保持する							*/
				/*----------------------------------------------------------*/
				if(status){
					RegistReceiveCommand(&command[0], LEN_COMMAND);
				}
			} else {
				/* 接続が切れた場合は再接続				*/
				if (close(socket) == -1) {
					ERRPrintf("[%s] failed to close(). errno: %d\n", __FUNCTION__, errno);
					errno = 0;
				}
				pDisconnectedFunc();
				SetSocketConnected(SOCKET_NULL);
				ret = 0;
																	DBGPrintf("[%s] TCPIPReceivedCommand length > 0:%d\n", __FUNCTION__, socket);
			}
		} else {
			/* システムエラー・接続しない				*/
			if (close(socket) == -1) {
				ERRPrintf("[%s] failed to close(). errno: %d\n", __FUNCTION__, errno);
				errno = 0;
			}
			pDisconnectedFunc();
			SetSocketConnected(SOCKET_NULL);
			ret = -1;
																	ERRPrintf("[%s] FD_ISSET Error\n", __FUNCTION__);
    	}
	} else {
		/* 受信タイムアウトが発生した場合は切断・再接続				*/
		if (close(socket) == -1) {
			ERRPrintf("[%s] failed to close(). errno: %d\n", __FUNCTION__, errno);
			errno = 0;
		}
		pDisconnectedFunc();
		SetSocketConnected(SOCKET_NULL);
		ret = 0;
																	DBGPrintf("[%s] TCPIPReceivedCommand select false:%d\n", __FUNCTION__, socket);
	}
	return ret;
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  SelectReceivePacket				                                        */
/*                                                                          */
/* 2.Description                                                            */
/* パケットを抽出する														*/
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  int32_t       0:コマンドなし  1 :コマンドあり	                        */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static int32_t SelectReceivePacket(uint8_t* pReceiveData, int32_t length, uint8_t* pCommand)
{
	int32_t		ii = 0;
	int32_t		ret = 0;
	uint8_t	 	lenOfCommand = 0;
																	DBGPrintf("SelectReceivePacket length %d\n",length);
	for(ii = 0; ii < length; ii++){

		if(pReceiveData[ii] & 0x80){						/* コマンド確定		*/
			lenOfCommand  = pReceiveData[ii+1];

			if(ii + lenOfCommand + TCP_HEADER_SIZE <= length){
				memcpy(pCommand, &pReceiveData[ii], lenOfCommand + TCP_HEADER_SIZE);
				ret = 1;
																	DBGPrintf("SelectReceivePacket OK %d\n",lenOfCommand);
				break;
			}
			else{
			/* コマンド長がサイズを超えていれば受信を無視する				*/
																	DBGPrintf("SelectReceivePacket NG2\n");
				break;
			}
		}
		else{
			/* コマンドと異なる受信をした場合は無視する						*/
																	DBGPrintf("SelectReceivePacket NG3\n");
		}
	}
	return ret;
}


/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  RegistReceiveCommand													*/
/*                                                                          */
/* 2.Description                                                            */
/* 受信コマンドを共有メモリに保持する										*/
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  uint8_t* data　　受信コマンド											*/
/*  uint32_t length　受信コマンドサイズ										*/
/*                                                                          */
/* 4.Return                                                                 */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static void RegistReceiveCommand(uint8_t* data, uint32_t length)
{
	if (AccessSemaphore(1) == 0) { // セマフォ取得
		memcpy(_receiveBuffer.receivedCommand[_receiveBuffer.num], data, (data[1] + TCP_HEADER_SIZE));
		_receiveBuffer.num++;
		memset(data, 0, length);
		AccessSemaphore(0); // セマフォ返還
	}
}

/*--------------------------------------------------------------------------*/
/* 1.Name                                                                   */
/*  SetSocketConnected				                                        */
/*                                                                          */
/* 2.Description                                                            */
/*  ソケット状態を設定する					                                */
/*                                                                          */
/* 3.Parameter(s)                                                           */
/*  None                                                                    */
/*                                                                          */
/* 4.Return                                                                 */
/*  None                                                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
static void SetSocketConnected(int32_t targetSocket)
{
	if (AccessSemaphore(1) == 0) { // セマフォ取得
		_targetSocket = targetSocket;
		AccessSemaphore(0); // セマフォ返還
	}
}
