/*
 *	doa.h
 */

//********************************************************************************
#pragma once

//********************************************************************************
#include <cmath>
#include <stdint.h>
#include <dsp_lib/include/dsp/dsp_module.h>
#include <dsp_lib/include/misc/dsp_const.h>
#include <dsp_lib/include/misc/null_check.h>

//********************************************************************************
namespace dsp_lib {

//********************************************************************************
class Doa : public DspModule {
	public:
		//----------------------------------------
		static const int	DOA_NUMBER_OF_SENSOR		= 3;
		static const int	DOA_NUMBER_OF_SENSOR_PAIR	= 2;

		//----------------------------------------
		typedef struct {
			uint32_t	sensor_ch[DOA_NUMBER_OF_SENSOR];		//!< sensor ch, pair0: [0][1], pair1: [1][2]
			float		sensor_pos_y[DOA_NUMBER_OF_SENSOR];		//!< sensor position [m]
		} Config;

		typedef struct {
			float		peak_threshold;							//!< Cxy peak threshold [dB]
			float		smooth_tc;								//!< time constant for smoothing [sec]
			float		source_pos_x_min;						//!< area-restriction [m]
			float		source_pos_x_max;						//!< area-restriction [m]
			float		source_pos_y_min;						//!< area-restriction [m]
			float		source_pos_y_max;						//!< area-restriction [m]
		} Param;

		typedef struct {
			float		peak[DOA_NUMBER_OF_SENSOR_PAIR];		//!< Cxy peak [dB]
			int32_t		lag[DOA_NUMBER_OF_SENSOR_PAIR];			//!< Cxy lag [sample]
			float		source_pos_x;							//!< estimated source position x [m]
			float		source_pos_y;							//!< estimated source position y [m]
		} Status;

		//----------------------------------------
		Doa(
			Config*			config_,
			Param*			param_,
			Status* 		status_
		);

		static bool Configure(
			Config*			config_,
			uint32_t*		sensor_ch_,
			float*			sensor_pos_y_
		)
		{
			if (NullCheck(config_, sensor_ch_, sensor_pos_y_) == false) {
				return false;
			}

			for (int ch=0; ch<DOA_NUMBER_OF_SENSOR; ch++) {
				config_->sensor_ch[ch] = sensor_ch_[ch];
				config_->sensor_pos_y[ch] = sensor_pos_y_[ch];
			}

			return true;
		}

		static bool Attach()
		{
			return true;
		}

		bool Ready();

		bool Execute(
			const float*	input_,
			float*			output_
		);

		bool Task();

	protected:
		//----------------------------------------
		static const int	DOA_FRAME_SIZE				= 512;

		//----------------------------------------
		typedef struct {
			float		y[2];													//!< focus
			float		d;														//!< difference of distance from y[0], y[1]
		} Hpbl;

		typedef struct {
			uint32_t	sensor_ch[DOA_NUMBER_OF_SENSOR];
			Hpbl		hpbl[DOA_NUMBER_OF_SENSOR_PAIR];
			float***	buf;													//!< [2][DOA_NUMBER_OF_SENSOR][DOA_BUF_SIZE*2]
			uint32_t	buf_size;												//!< must be multiple of DSP_BLOCK_SIZE & greater than DOA_FRAME_SIZE
			uint32_t	buf_point;
			uint32_t	buf_pingpong;
			uint32_t	buf_full_flg;
			uint32_t	buf_cnt;
			float		pos_x_rectified;
			float		pos_y_rectified;
		} Data;

		//----------------------------------------
		static float	s_doa_w[DOA_FRAME_SIZE];								//!< twiddle factor
		static float	s_doa_win[DOA_FRAME_SIZE];								//!< for POC, window
		static float	s_doa_rabs[DOA_NUMBER_OF_SENSOR][DOA_FRAME_SIZE];		//!< for POC, 1/abs()

		//----------------------------------------
		void GetCxy(
			float*			X_,
			float*			Y_,
			float*			Cxy_,
			const int		ndft_
		);
		void GetMax(
			const float*	x_,
			const int		nx_,
			float*			maxval_,
			int*			maxidx_
		);
		void GetHpblCp(
			const Hpbl		hpbl1_,
			const Hpbl		hpbl2_,
			float*			pos_x_,
			float*			pos_y_
		);

		//----------------------------------------
		Config*	m_config;
		Param*	m_param;
		Status*	m_status;

		Data	m_data;
};

//********************************************************************************
}	// namespace dsp_lib
