/*
 * dsp_nc.h
 *
 *  Created on: 2017/01/30
 *      Author: toa29548
 */

#pragma once

/*!
 * Noise Canceler
 */

#include <stdint.h>
#include <dsp_lib/include/misc/dsp_const.h>
#include <dsp_lib/include/dsp/dsp_module.h>

namespace dsp_lib {

class NoiseEstimator;

class Nc : public DspModule {
public:
	// -------------------------------------------------------------
	// 定数
	// -------------------------------------------------------------
	/*!
	 * @brief	Noise Canceler アルゴリズム
	 */
	enum NC_ALG
	{
		/*!
		 * - Spectral Subtraction [Boll 1979]
		 * - ミュージカルノイズが出やすい。
		 */
		SS = 0,

		/*!
		 * - Wiener Filter [B.Widrow, J.G.R.Glover, 1975]
		 * - ミュージカルノイズが出にくい。
		 */
		WF = 1,

		/*!
		 * - Minimum Mean Square Error Short Time Spectral Amplitude Estimator [Ephraim 1984]
		 * - WFの発展形
		 */
		STSA = 2,

		/*!
		 * - Minimum Mean Square Error Short Time Log Spectral Amplitude Estimator [Ephraim 1985]
		 * - STSAの発展形
		 */
		LSA = 3,

		/*!
		 * - Maximum A Posteriori Spectral Amplitude Estimator [P.J.Wolf 2003]
		 * - LSAよりも音声はやや小さくなるがミュージカルノイズも小さくなる
		 */
		MAP = 4
	};

	/*!
	 * @brief	Minimum Statistics アルゴリズム
	 */
	enum NEST_MODE
	{
		/*!
		 * - Normal
		 * - 単純な平滑化による雑音推定。
		 */
		RECT = 0,

		/*!
		 * - Minimum Statistics
		 * - 最小統計量。音声劣化が軽減される。
		 */
		MS = 1
	};

static const int NC_NEST_BAND_NUM	= 2;					//!< NEST:雑音推定の帯域数
static const int NC_MS_NUM_MAX		= 16;					//!< MS:最小統計量。推定雑音スペクトルを保持する数。2のべき乗。

//static const int NC_BLOCK_SIZE	= DSP_BLOCK_SIZE;
static const int NC_BLOCK_SIZE	= 512;

// Frame Configuration (全て2のべき乗であること)
static const int SHIFT_LEN 		= NC_BLOCK_SIZE;			//!< @brief 周波数分析フレームのシフト長。DSP_BLOCK_SIZEと合わせること。
static const int FRAME_NUM   	= 2;						//!< @brief フレーム合成時に重ね合わせるフレーム数。2の場合50%, 4の場合25%オーバーラップとなる。
static const int FRAME_LEN 		= FRAME_NUM * SHIFT_LEN;	/*!< @brief 周波数分析フレーム長。長いほど音質が向上するが、遅延が増える。
															 *   - default = 512
															 *   - min, max = 256, 1024*/

static const float SHIFT_FS;			//!< @brief 周波数分析の頻度（Hz）

	// -------------------------------------------------------------
	// データ型
	// -------------------------------------------------------------

	/*!
	 * @brief NCワンショット構造体
	 * - 一度DSPに引き渡すと0に戻る
	 */
	typedef struct{
		uint32_t	reset;
	} nc_oneshot;

	// -----------------------------

	struct Config{};
	struct Param{
		uint32_t	bypass;							//!< 信号処理をバイパス\n　1:bypass, 0:not bypass
		float 		prisnr_ta; 						//!< 事後SNR推定のアタックタイム（増加時）[sec]
		float 		prisnr_tr; 						//!< 事後SNR推定のアタックタイム（減少時）[sec]
		float		flr_db;							//!< ノイズフロア
		float		ss_subt_coef;					//!< SS:抑圧係数
		uint32_t	alg;							//!< 雑音除去アルゴリズム, NC_ALGから選択
		nc_oneshot	oneshot;
	};
	struct Status{};

	// -------------------------------------------------------------
	// コンフィギュレーション（実行時に変化しないパラメータ設定）
	// -------------------------------------------------------------
	static bool Configure(void) {
		return true;
	}
	static bool Attach(void) {
		return true;
	}

	Nc(
			Config*	/*config_*/,
			Param*	param_,
			Status*	/*status_*/
	);

	bool Ready(void);

	bool Execute(
			const float*	input_,
				  float*	output_
	);
	bool Task(void);

protected:
	static const float	FAST_NOISE_ESTIMATION_DURATION;			//!< 高速雑音推定を行う期間[sec]
	static const float	FAST_NOISE_ESTIMATION_ATTACK_TIME;		//!< 高速雑音推定時のアタックタイム[sec]
	static const float	FAST_NOISE_ESTIMATION_RELEASE_TIME;		//!< 高速雑音推定時のリリースタイム[sec]

	static const int NC_BUFFER_SIZE;			//!< DSP_BLOCK_SIZE と NC_BLOCK_SIZE の最小公倍数
	static const float PSD_MAX;					//!< パワースペクトルの最大値

	static float	s_window[FRAME_LEN];		//!< 窓関数

	static float	s_psd[FRAME_LEN/2];			//!< 入力信号のパワースペクトル
	static float 	s_nestuse[FRAME_LEN/2];		//!< 事後SNR算出に用いる推定雑音スペクトル
	static float 	s_prisnr[FRAME_LEN/2];		//!< 事前SNR

	static uint32_t	s_window_initialized;		//!< nc_comの初期化完了フラグ

	typedef struct{
		float		frame[FRAME_LEN*2];				//!< 入力のフレーム分割に使用
		float		wsyn[FRAME_NUM][SHIFT_LEN];		//!< 出力のフレーム合成に使用
		float		gain[FRAME_LEN/2];				//!< ゲインスペクトル
		float 		postsnr[2][FRAME_LEN/2];		//!< 事後SNR
		int			fidx;							//!< フレームのインデックス
		int 		postsnridx;						//!< postsnrのインデックス
		int			fast_nest_cnt;					//!< 高速雑音推定カウンタ, NC使用開始直後のみ高速に適応するために使用
	} nc_data;

	void nc_initialize_data(void);
	void nc_dft(const float *ibuf_);
	void nc_idft_and_frame_synthesis(float* const obuf_);
	void nc_power_spectrum_calculate(void);
	void nc_nest(
			float *nest_, float *psd_,
			float ta_, float tr_, float tr_coef_,
			int fmin_, int fmax_, int sflag_
	);
	void nc_PosterioriSNREstimation(void);
	void nc_PrioriSNREstimation(void);
	void nc_speech_spectrum_estimation(void);
	void nc_through(const float * const ibuf_, float* const obuf_);
	void nc_through_dft(float* const obuf_, const float * const ibuf_);
	void nc_initialize_param(void);

	void nc_buffer_input(const float* input_);
	void nc_buffer_output(float* output_);
	void nc_reset_index(void);
	void nc_execute(const float* input_,	float* output_);

	//	Config*		m_config;
		Param*		m_param;
	//	Status*		m_status;

		float*		m_ibuf;
		float*		m_obuf;

		uint32_t	m_ibuf_widx;			//!< 入力バッファの書き込みインデックス
		uint32_t	m_ibuf_ridx;			//!< 入力バッファの読み出しインデックス
		uint32_t	m_ibuf_readable_num;	//!< 入力バッファの読み出し可能サンプル数
		uint32_t	m_obuf_widx;			//!< 出力バッファの書き込みインデックス
		uint32_t	m_obuf_ridx;            //!< 出力バッファの読み出しインデックス
		uint32_t	m_obuf_readable_num;    //!< 出力バッファの読み出し可能サンプル数

		nc_data		m_data;

		NoiseEstimator*	m_noise_estimator;

};

} //namespace dsp_lib
