/*
 * simulated_malloc.cpp
 *
 *  Created on: 2017/03/15
 *      Author: toa29548
 */

#include <stdlib.h>
#include <fstream>
#include <iostream>
#include <iomanip>
#include "simulated_malloc.h"
#include <shared/shared_ram.h>

using namespace std;

namespace dsp_ctrl {

const uint32_t LOCALRAM_ADDRESS_UNIT_BYTE = 1;
const uint32_t ADDRESS_UNIT_RATIO = SHAREDRAM_ADDRESS_UNIT_BYTE / LOCALRAM_ADDRESS_UNIT_BYTE;
const uint32_t DSP_DM_WORD_ALIGN = 8;	// byte
const uint32_t DSP_PM_WORD_ALIGN = 8;	// byte

struct heap_info {
	uint32_t length;		// word
	uint32_t used;			// word
	uint32_t unused;		// word
	uint32_t byte_in_word;	// word(アドレス1番地)あたりのbyte
	uint32_t word_align;	// 割り当て時の先頭番地をこの値の倍数にする
	uint32_t base_address;	// heap領域の先頭番地
	uint32_t end_address;	// heap領域の末端の番地
	uint32_t last_address;	// 最後に割り当てた領域の先頭番地
	uint32_t next_address;	// 次に割り当てる領域の先頭番地
};

heap_info g_dm_heap_info;
heap_info g_pm_heap_info;

void*		g_heap_base_origin = NULL;

uint32_t	g_local_heap_address_base		= 0;
uint32_t	g_local_heap_address_boot_info	= 0;
uint32_t	g_local_heap_address_map		= 0;
uint32_t	g_local_heap_address_dm			= 0;
uint32_t	g_local_heap_address_pm			= 0;

void
print_heap_info(heap_info &heap_, ostream &ros) {
	ros << "length:       0x"	<< hex << setw(8) << setfill('0') << heap_.length		<< " word" << endl;		
	ros << "used:         0x"	<< hex << setw(8) << setfill('0') << heap_.used			<< " word" << endl;
	ros << "unused:       0x"	<< hex << setw(8) << setfill('0') << heap_.unused		<< " word" << endl;
	ros << "byte_in_word: 0x" 	<< hex << setw(8) << setfill('0') << heap_.byte_in_word	<< endl;
	ros << "word_align:   0x" 	<< hex << setw(8) << setfill('0') << heap_.word_align	<< endl;	
	ros << "base_address: 0x" 	<< hex << setw(8) << setfill('0') << heap_.base_address	<< endl;
	ros << "end_address:  0x" 	<< hex << setw(8) << setfill('0') << heap_.end_address	<< endl;
	ros << "last_address: 0x" 	<< hex << setw(8) << setfill('0') << heap_.last_address	<< endl;
	ros << "next_address: 0x" 	<< hex << setw(8) << setfill('0') << heap_.next_address	<< endl;
}

uint32_t
align_2n(
	uint32_t	x_,
	uint32_t	n2_	//!< 任意の偶数
)
{
	uint32_t x = x_;
	x += (n2_ - 1);
	x &= ~(n2_ - 1);	// 端数を切り捨て
	return x;
}

uint32_t
byte_to_word(
	uint32_t	byte_,
	uint32_t	byte_in_word
)
{
	uint32_t	word = align_2n(byte_, byte_in_word) / byte_in_word;
	return word;
}

void
init_heap_info(
	heap_info* heap_, 
	uint32_t base_address_,
	uint32_t length_, 
	uint32_t byte_in_word_,
	uint32_t word_align_
	)
{
	uint32_t heap_base;
	heap_base = base_address_;
	heap_base = align_2n(heap_base, word_align_);

	heap_->length		= length_;
	heap_->used			= 0;
	heap_->unused		= length_;
	heap_->byte_in_word	= byte_in_word_;
	heap_->word_align	= word_align_;
	heap_->base_address	= heap_base;
	heap_->end_address	= heap_base + length_ - 1;
	heap_->last_address	= heap_base;
	heap_->next_address	= heap_base;
}

uint32_t
addressing_convert_dsp2cpu(uint32_t dsp_address_)
{
	return dsp_address_ * ADDRESS_UNIT_RATIO;
}

uint32_t
addressing_convert_cpu2dsp(uint32_t cpu_address_)
{
	return cpu_address_ / ADDRESS_UNIT_RATIO;
}

bool
InitializeLocalHeap(void)
{
	// malloc結果をアラインするため、heapを余分にとっておく
	uint32_t heap_byte = addressing_convert_dsp2cpu(SHAREDRAM_LENGTH) + DSP_DM_WORD_ALIGN;
	g_heap_base_origin = malloc(heap_byte);

	if (g_heap_base_origin == NULL) {
		return false;
	}

	for (uint32_t i = 0; i < heap_byte; i++) {
		reinterpret_cast<uint8_t*>(g_heap_base_origin)[i] = 0;
	}

	uint32_t base = align_2n(reinterpret_cast<uint32_t>(g_heap_base_origin), DSP_DM_WORD_ALIGN);

	g_local_heap_address_base		= base;
	g_local_heap_address_boot_info	= base + addressing_convert_dsp2cpu(SHAREDRAM_POSITION_BOOT_INFO);
	g_local_heap_address_map		= base + addressing_convert_dsp2cpu(SHAREDRAM_POSITION_MAP);
	g_local_heap_address_dm			= base + addressing_convert_dsp2cpu(SHAREDRAM_POSITION_DM);
	g_local_heap_address_pm			= base + addressing_convert_dsp2cpu(SHAREDRAM_POSITION_PM);

	init_heap_info(
		&g_dm_heap_info,
		g_local_heap_address_dm,
		addressing_convert_dsp2cpu(SHAREDRAM_DM_LENGTH),
		LOCALRAM_ADDRESS_UNIT_BYTE,
		DSP_DM_WORD_ALIGN
		);

	init_heap_info(
		&g_pm_heap_info,
		g_local_heap_address_pm,
		addressing_convert_dsp2cpu(SHAREDRAM_PM_LENGTH),
		LOCALRAM_ADDRESS_UNIT_BYTE,
		DSP_PM_WORD_ALIGN
		);

	return true;
}

#if 0 // on DSP
void*
simulated_malloc(
		size_t 		size_,
		uint32_t&	stored_address_,
		uint32_t	end_address_,
		uint32_t	word_align_
)
{
	void*		ptr_return		= NULL;
	uint32_t	current_address	= stored_address_;
	uint32_t	next_address	= current_address + size_;

	// alignment
	next_address += word_align_ - 1;
	next_address &= ~(word_align_ - 1);	// 端数を切り捨て

	// メモリが足りない場合はNULLを返す
	if (next_address > (end_address_ + 1))
	{
		return NULL;
	}

	stored_address_ = next_address;

	ptr_return = reinterpret_cast<void*>(current_address);

	return ptr_return;
}
#else
uint32_t
dev_ceilu(
	uint32_t numerator_,
	uint32_t denominator_
)
{
	return (numerator_ + (denominator_ - 1))/denominator_;
}


void*
simulated_malloc(
		size_t 		size_byte_,
		heap_info*	heap_
)
{
	uint32_t	current_address	= heap_->next_address;
	uint32_t	next_address	= current_address + dev_ceilu(size_byte_, heap_->byte_in_word);	// 切り上げ
	uint32_t	word_align		= heap_->word_align;
	uint32_t	end_address		= heap_->end_address;

	// alignment
	next_address += word_align - 1;
	next_address &= ~(word_align - 1);	// 端数を切り捨て

	// メモリが足りない場合はNULLを返す
	if (next_address > (end_address + 1))
	{
		return NULL;
	}

	heap_->last_address = current_address;
	heap_->next_address = next_address;
	heap_->used			+= heap_->next_address - heap_->last_address;
	heap_->unused		= heap_->length - heap_->used;

	return reinterpret_cast<void*>(current_address);
}
#endif
void*
simulated_malloc_dm(uint32_t size_byte_)
{
	void*	ptr_return;
	ptr_return = simulated_malloc(size_byte_, &g_dm_heap_info);

	return ptr_return;
}

void*
simulated_malloc_pm(uint32_t size_byte_)
{
	void*	ptr_return;
	ptr_return = simulated_malloc(size_byte_, &g_pm_heap_info);

	return ptr_return;
}

void
print_heap_infos(ostream &ros)
{
	ros << "[SHAREDRAM DM HEAP]" << endl;
	print_heap_info(g_dm_heap_info, ros);
	ros << endl;
	ros << "[SHAREDRAM PM HEAP]" << endl;
	print_heap_info(g_pm_heap_info, ros);
}

void
PrintHeapInfo(void)
{
	cout << endl;
	print_heap_infos(cout);
	cout << endl;
}

bool
SaveHeapInfo(void)
{
	FILE *fp;
	std::string filename = "./log_heap_info.txt";
	ofstream ofs(filename);
	if (!ofs) {
		cout << "file: " << filename << " cannot be opened!" << endl;
		return false;
	}

	print_heap_infos(ofs);

	return true;
}

} // namespace dsp_ctrl
