/*
 *	bf.h
 */

//********************************************************************************
#pragma once

//********************************************************************************
#include <stdint.h>
#include <dsp_lib/include/dsp/delay.h>
#include <dsp_lib/include/dsp/dsp_module.h>
#include <dsp_lib/include/misc/dsp_const.h>
#include <dsp_lib/include/misc/null_check.h>
#include "fir.h"

//********************************************************************************
namespace dsp_lib {

//********************************************************************************
class Bf : public DspModule {
	public:
		//----------------------------------------
		static const int	BF_SIZE_OF_SUBARRAY = 3;

		//----------------------------------------
		typedef int32_t	SubarrayCh[BF_SIZE_OF_SUBARRAY];

		typedef struct {
			uint32_t	number_of_ch;							//!<
			float*		sensor_pos_y;							//!< [m]								[NUMBER_OF_MIC]
			SubarrayCh*	subarray_ch;							//!<									[NUMBER_OF_MIC-1]
		} Config;

		typedef struct {
			float		focus_x;								//!< [m]
			float		focus_y;								//!< [m]
		} Param;

		typedef struct {
			float		focus_x;								//!< [m]
			float		focus_y;								//!< [m]
		} Status;

		//----------------------------------------
		Bf(
			Config*			config_,
			Param*			param_,
			Status* 		status_
		);

		static bool Configure(
			Config*			config_,
			uint32_t		number_of_ch_,
			float*			sensor_pos_y_,
			SubarrayCh*		subarray_ch_
		)
		{
			if (NullCheck(config_, sensor_pos_y_, subarray_ch_) == false) {
				return false;
			}
			if (number_of_ch_ <= 0) {
				return false;
			}

			config_->number_of_ch = number_of_ch_;
			config_->sensor_pos_y = sensor_pos_y_;
			config_->subarray_ch = subarray_ch_;

			return true;
		}

		static bool Attach()
		{
			return true;
		}

		bool Ready();

		bool Execute(
			const float*	ibuf_,
			float*			obuf_
		);

		bool Task();

	protected:
		//----------------------------------------
		static const int	NDFT		= 128;
		static const int	FIR_LENGTH	= NDFT;					//!< FIR_LENGTH <= NDFT

		//----------------------------------------
		typedef float	Io[DSP_BLOCK_SIZE];
		typedef float	Bpf[NDFT/2+1];

		typedef struct {
			Io*			io;										//!<									[NUMBER_OF_CH]
			float		update_busy;							//!<
			Bpf*		bpf;									//!< 									[NUMBER_OF_CH-1]
			float*		fc;										//!< center frequency [Hz]				[NUMBER_OF_CH-1]
			float*		weight;									//!< 									[NUMBER_OF_CH-1][NUMBER_OF_CH]
			float*		delay_decimal;							//!< delay [sample]						[NUMBER_OF_CH]
			float*		workarea_ch_x_1;						//!<									[NUMBER_OF_CH]
		} Data;

		//----------------------------------------
		static float	s_bf_w[NDFT];							//!< twiddle factor

		//----------------------------------------
		uint32_t CalcDelayBufSize();
		void CalcDelay();
		void UpdateDelay();
		void UpdateFir();
		void Update();
		void InitializeWeight();
		void InitializeBpf();
		void InitializeDelay();

		//----------------------------------------
		Param*			m_param;
		Status*			m_status;

		uint32_t    	m_number_of_ch;
		float*			m_sensor_pos_y;
		SubarrayCh*		m_subarray_ch;

		Data			m_data;

		Delay**			m_delay;
		Delay::Param*	m_delay_param;

		Fir**			m_fir;
		Fir::Param*		m_fir_param;
};

//********************************************************************************
}	// namespace dsp_lib
