/*
 *	dsp_iir.h
 *
 */
#pragma once

#include <stdint.h>
#include <dsp_lib/include/misc/null_check.h>
#include <dsp_lib/include/dsp/dsp_module.h>
#include <dsp_lib/misc/biquad/fdesign.h>

namespace dsp_lib {

class Iir : public DspModule {
public:
	// -------------------------------------------------------------
	// 定数
	// -------------------------------------------------------------
	//#if defined TI_OMAPL138
	//const int SIZE_OF_IIR_COEF = 3+3;	// IIRフィルタ係数のパラメータ数
	//const int SIZE_OF_IIR_BUFFER = 2;	// IIRフィルタのバッファ長, DSPF_sp_biquad()が直接型II転置構成なので2で固定。
	//const int COEF_POSITION_B0 = 0;
	//const int COEF_POSITION_A0 = 3;
	//#elif defined ADI_ADSP21489
	//const int SIZE_OF_IIR_COEF = 3+2;	// IIRフィルタ係数のパラメータ数
	//const int SIZE_OF_IIR_BUFFER = 2+1;	// IIRフィルタのバッファ長, 2*sections + 1。
	//const int COEF_POSITION_B0 = 4;
	//#else
	//#endif
	static const int SIZE_OF_IIR_COEF = 6;		// IIRフィルタ係数のパラメータ数, 最大のデバイスに合わせる
	static const int SIZE_OF_IIR_BUFFER = 2+1;	// IIRフィルタのバッファ長, 2*sections + 1, 最大のデバイスに合わせる

	// -------------------------------------------------------------
	// データ型
	// -------------------------------------------------------------
	typedef float IirCoef[SIZE_OF_IIR_COEF];
	typedef float IirBuffer[SIZE_OF_IIR_BUFFER];

	struct Config{
		uint32_t		number_of_stage;
	};
	struct Param {
		IirCoef pm *	coef;				//!< 外部で用意したcoef[stage][SIZE_OF_IIR_COEF]の先頭番地を指定
		uint32_t*		coef_update_req;	//!< 外部で用意したcoef_update_req[stage]の先頭番地を指定
	};
	struct Status{};

	// -------------------------------------------------------------
	// コンフィギュレーション（実行時に変化しないパラメータ設定）
	// -------------------------------------------------------------

	/*!
	 * Matrixコンフィギュレーションの初期化
	 * @note
	 * @param config_ コンフィギュレーション構造体
	 * @param number_of_stage_ biquadのカスケード数
	 */
	static bool
	Configure(
	    Config*    config_,
	    uint32_t   number_of_stage_
	)
	{
		if (NullCheck(config_) == false) {
			return false;
		}
		if (number_of_stage_ <= 0) {
			return false;
		}

		config_->number_of_stage = number_of_stage_;

		return true;
	}

	/*!
	 * 可変長データアドレスの登録
	 * @note
	 * @param param_ パラメーター構造体
	 * @param coef_ biquad係数として登録する領域の先頭番地。この番地からnumber_of_stage_ * SIZE_OF_IIR_COEF * 4 byte を使用する
	 */
	static bool
	Attach(
	    Param*			param_,
		IirCoef pm *	coef_,
		uint32_t*		coef_update_req_
	)
	{
		if (NullCheck(param_) == false) {
			return false;
		}

		if (coef_ == NULL) {
			return false;
		}
		if (coef_update_req_ == NULL) {
			return false;
		}

		param_->coef 		= coef_;
		param_->coef_update_req = coef_update_req_;

		return true;
	}

	// -------------------------------------------------------------
	// パラメータ変更
	// -------------------------------------------------------------
	/*!
	 * @brief フィルター設計ルーチン
	 * @param filtertype_	フィルタータイプ(enum FILTER_TYPE から選択)
	 * @param f				周波数
	 * @param gain			ゲイン
	 * @param q				Q
	 * @param alphaindex_	alpha
	 * @param eqon_			on/off
	 * @param coeff_		フィルター係数
	 */
	static bool
	SetIirCoeff(
			int32_t		filtertype_,
			float		f,
			float		gain,
			float		q,
			int32_t		alphaindex_,
			int32_t		eqon_,
			IirCoef	pm*	coeff_
	){
		float coeff[6];

		if (NullCheck(coeff_) == false) {
			return false;
		}

		Fdesign(
			filtertype_,
			f,
			gain,
			q,
			alphaindex_,
			eqon_,
			coeff
			);

		float*	before = coeff;
		float*	after = reinterpret_cast<float*>(coeff_);

		// [before]
		//	    	b0 + b1*z^-1 + b2*z^-2
		// H(z) =	-----------------------
		//    		1 + a1*z^-1 + a2*z^-2

		// [after]
		//	    	b0 + b1*z^-1 + b2*z^-2
		// H(z) =	-----------------------
		//    		1 - a1*z^-1 - a2*z^-2

		after[0] = -before[4];	// a2
		after[1] = -before[3];	// a1
		after[2] = before[2];		// b2
		after[3] = before[1];		// b1
		after[4] = before[0];		// b0

		return true;
	};

public:
	Iir(
			Config*	config_,
			Param*	param_,
			Status*	/*status_*/
	);

	bool Ready(void);

	bool Execute(
			const float*	input_,
				  float*	output_
	);
	bool Task(void);

protected:
	void GetIirCoeff(void);

	Config*		m_config;
	Param*		m_param;

	uint32_t	m_number_of_stage;

	IirBuffer*	m_buf;				//!< 外部で用意したbuf[stage][SIZE_OF_IIR_BUFFER]の先頭番地を指定
	IirCoef pm*	m_coe;				//!< 外部で用意したcoe[stage][SIZE_OF_IIR_COEF]の先頭番地を指定
};

} //namespace dsp_lib
