/*
 *	matrix.h
 *
 *  Created on: 2016/09/16
 *      Author: toa29548
 */
#pragma once

#include <stdint.h>
#include <dsp_lib/include/misc/null_check.h>
#include <dsp_lib/include/dsp/dsp_module.h>

namespace dsp_lib {

class Matrix : public DspModule {
public:

	// -------------------------------------------------------------
	// データ型
	// -------------------------------------------------------------
	// 処理に必要なデータサイズを決める設定項目, 初期化時のみ使用
	struct Config {
	    uint32_t    number_of_input;	//!< 入力チャンネル数
	    uint32_t    number_of_output;	//!< 出力チャンネル数
	};

	struct Param {
	    float*      gain_target;		//!< ゲインの目標値[真数]
	    float       attack_time;		//!< ゲインのアタックタイム
	    float       release_time;		//!< ゲインのリリースタイム
	};

	struct Status {};

	// -------------------------------------------------------------
	// コンフィギュレーション（実行時に変化しないパラメータ設定）
	// -------------------------------------------------------------

	/*!
	 * Matrixコンフィギュレーションの初期化
	 * @note
	 * @param config_ コンフィギュレーション構造体
	 * @param number_of_input_ Matrixの入力チャンネル数
	 * @param number_of_output_ Matrixの出力チャンネル数
	 */
	static bool
	Configure(
	    Config*    config_,
	    uint32_t   number_of_input_,
	    uint32_t   number_of_output_
	)
	{
		if (NullCheck(config_) == false) {
			return false;
		}
		if (number_of_input_ <= 0) {
			return false;
		}
		if (number_of_output_ <= 0) {
			return false;
		}

		config_->number_of_input = number_of_input_;
		config_->number_of_output = number_of_output_;

		return true;
	}

	/*!
	 * 可変長データアドレスの登録
	 * @note
	 * @param param_ パラメーター構造体
	 * @param gain_target_ gain_targetとして登録する領域の先頭番地。この番地からnumber_of_input_ * number_of_output_ * 4 byte を使用する
	 */
	static bool
	Attach(
	    Param*    param_,
	    float*    gain_target_
	)
	{
		if (NullCheck(param_, gain_target_) == false) {
			return false;
		}

		param_->gain_target 		= gain_target_;

		return true;
	}

	// -------------------------------------------------------------
	// パラメータ変更
	// -------------------------------------------------------------
	/*!
	 *
	 * @param config_ Matrixのコンフィグ構造体
	 * @param param_ Matrixのパラメータ構造体
	 * @param input_no_ 入力元チャンネル番号
	 * @param output_no_ 出力先チャンネル番号
	 * @param gain_ クロスポイントゲイン[真数]
	 * @return
	 */
	static bool
	SetGain(
		Config*		config_,
		Param*		param_,
		uint32_t	input_no_,
		uint32_t	output_no_,
		float		gain_
	)
	{
		if (NullCheck(config_, param_) == false) {
			return false;
		}

		uint32_t number_of_input	= config_->number_of_input;
		uint32_t number_of_output	= config_->number_of_output;
		uint32_t index				= input_no_	+ output_no_ * number_of_input;

		if (index < number_of_input * number_of_output)
		{
			param_->gain_target[index] = gain_;
		}

		return true;
	}

	/*!
	 * Matrixの時定数を設定する
	 *
	 * @param Param Matrixのパラメーター構造体
	 * @param attack_time_ ゲイン変化のアタックタイム[sec]
	 * @param release_time_ ゲイン変化のリリースタイム[sec]
	 */
	static bool
	SetTimeFactor(
		Param*	param_,
		float	attack_time_,
		float	release_time_
	)
	{
		if (NullCheck(param_) == false) {
			return false;
		}
		param_->attack_time			= attack_time_;
		param_->release_time		= release_time_;

		return true;
	}

public:
	Matrix(
			Config*	config_,
			Param*	param_,
			Status*	/*status_*/
	);

	bool Ready(void);
	bool Execute(
			const float*	input_,
			float*			output_
	);
	bool Task(void);

protected:
	void
	execute_smoothing(
		Param*	param_
	);

	void
	execute_matrix(
		float*	input_,
		float*	output_,
		float*	gain_,
		uint32_t	number_of_input_,
		uint32_t	number_of_output_
	);

	Config*		m_config;
	Param*		m_param;
//	Status*		m_status;

	uint32_t    m_number_of_input;    //!< Config構造体から取り込む
	uint32_t    m_number_of_output;   //!< Config構造体から取り込む

	float*		m_gain_current;		//!< ゲインの現在値[真数]
	float		m_attack_ff;			//!< ゲイン上昇時の忘却係数
	float		m_release_ff;         //!< ゲイン下降時の忘却係数
};

} //namespace dsp_lib
